/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __TIZEN_CAPI_NETWORK_USER_AWARENESS_PRIVATE_H__
#define __TIZEN_CAPI_NETWORK_USER_AWARENESS_PRIVATE_H__


#include <glib.h>
#include <user-awareness-log.h>
#include <user-awareness.h>

#include <system_info.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief Returns if expr is true.
 * @since_tizen 6.5
 */
#define ret_if(expr) \
	do { \
		if (expr) { \
			UA_DBG("(%s) return", #expr); \
			return; \
		} \
	} while (0)

/**
 * @brief Returns val if expr is true.
 * @since_tizen 6.5
 */
#define retv_if(expr, val) \
	do { \
		if (expr) { \
			UA_DBG("(%s) return", #expr); \
			return (val); \
		} \
	} while (0)

/**
 * @brief Returns error code If not initialized.
 * @since_tizen 6.5
 */
#define UA_CHECK_INIT_STATUS() \
	retv_if(TRUE != _ua_is_initialized(), UA_ERROR_NOT_INITIALIZED);

/**
 * @brief Returns error code if expr is NULL.
 * @since_tizen 6.5
 */
#define UA_VALIDATE_INPUT_PARAMETER(arg) \
	if (arg == NULL) { \
		UA_ERR("INVALID_PARAMETER(%s is NULL)", #arg); \
		return UA_ERROR_INVALID_PARAMETER; \
	}

/**
 * @brief Returns error code if expr is NULL.
 * @since_tizen 6.5
 */
#define UA_IS_EXIST(arg) \
	if (arg == NULL) { \
		UA_ERR("UA_ERROR_NOT_FOUND(%s is NULL)", #arg); \
		return UA_ERROR_NOT_FOUND; \
	}

/**
 * @brief Prints arg.
 * @since_tizen 6.5
 */
#define UA_PRINT_DEVICE_HANDLE(arg) \
		UA_INFO("Device Handle [%p]", #arg); \

/**
 * @brief Prints arg.
 * @since_tizen 6.5
 */
#define UA_PRINT_PAYLOAD_HANDLE(arg) \
		UA_INFO("Payload Handle [%p]", #arg); \

/**
 * @brief Prints arg.
 * @since_tizen 6.5
 */
#define UA_PRINT_USER_HANDLE(arg) \
		UA_INFO("User Handle [%p]", #arg); \

/**
 * @brief Checks if h1 is not in the list.
 * @since_tizen 6.5
 */
#define UA_VALIDATE_HANDLE(h1, list) \
	{ \
		GSList *l; \
		bool valid = FALSE; \
		for (l = list; l; l = g_slist_next(l)) { \
			void *h2 = (void *)l->data; \
			if (h1 == h2) { \
				UA_INFO("Handle matched [%p]", h2); \
				valid = TRUE; break; \
			} \
		} \
		if (valid == FALSE) { \
			UA_ERR("Handle [%p] did not match with any stored handles!!", h1); \
			return UA_ERROR_INVALID_PARAMETER; \
		} \
	} \

/**
 * @brief Returns true if h1 is in the list, else return false.
 * @since_tizen 6.5
 */
#define UA_EXIST_HANDLE(h1, list) \
	{ \
		GSList *l; \
		bool valid = FALSE; \
		for (l = list; l; l = g_slist_next(l)) { \
			void *h2 = (void *)l->data; \
			if (h1 == h2) { \
				UA_INFO("Handle matched [%p]", h2); \
				valid = TRUE; break; \
			} \
		} \
		if (valid == TRUE) { \
			UA_ERR("Handle [%p] match with any stored handles!!", h1); \
			return UA_ERROR_ALREADY_DONE; \
		} \
	} \

typedef enum {
	UA_FEATURE_BASE = 0,
	UA_FEATURE_COMMON = UA_FEATURE_BASE,
	UA_FEATURE_LOCATION,
	UA_FEATURE_MAX
} ua_feature_t;

typedef struct {
	const char *name;
	gboolean is_check;
	gboolean value;
} ua_feature_table_t;

int _ua_check_supported_feature(ua_feature_t feature, bool *supported);

#define UA_CHECK_SUPPORTED_FEATURE(feature) \
do	{ \
	bool is_supported = false; \
	if (!_ua_check_supported_feature(feature, &is_supported)) { \
		if (is_supported == false) { \
			UA_ERR("[%s] NOT_SUPPORTED", __FUNCTION__); \
			return UA_ERROR_NOT_SUPPORTED; \
		} \
	} else { \
		UA_ERR("[%s] Fail to get the system feature: [%d]", __FUNCTION__, feature); \
	} \
} while (0)


#define UA_ALL_SENSOR_PRESENCE_DETECTION_WAIT_TIME 65 /**< All sensor presence detection wait time */

/**
 * @brief Minimum detection window size.
 * @since_tizen 6.5
 */
#define UA_MAX_DETECTION_WINDOW 55 /**< 55 seconds */

/**
 * @brief Minimum detection cycle.
 * @since_tizen 6.5
 */
#define UA_MIN_DETECTION_CYCLE 60 /**< 1 min */

/**
 * @brief Maximum sensor values count.
 * @since_tizen 6.5
 */
#define UA_SENSOR_MAX_VALUES 4 /**< 4 values */

/**
 * @brief Default service name.
 * @since_tizen 6.5
 */
#define UA_SERVICE_DEFAULT "ua.service.default" /**< Default service name */

/**
 * @brief Device's BLE payload DEVICE_UID length.
 * @since_tizen 6.5
 */
#define UA_BLE_PAYLOAD_DEVICE_UID_MAX_LEN 17

/**
 * @brief The handle of payload information.
 * @since_tizen 6.5
 */
typedef void *ua_payload_h;

/**
 * @brief Event type for user event callback data structure.
 * @since_tizen 6.5
 */
typedef enum {
	UA_USER_EVENT_DEVICE_ADDED = 0, /**< Device added for a user */
	UA_USER_EVENT_MAX /**< Event Max. */
} ua_user_event_e;

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Enumerations of device presence state.
 * @since_tizen 6.5
 */
typedef enum {
	UA_PRESENCE_STATE_INACTIVATE = 0x00, /**< User state is inactive */
	UA_PRESENCE_STATE_ACTIVATE, /**< User state is active */
	UA_PRSENCE_STATE_INVALID = 0xFF /**< User state invalid */
} ua_presence_state_e;

/**
 * @brief Callback data structure.
 * @since_tizen 6.5
 */
typedef struct {
	const void *callback; /**< Callback function pointer */
	void *user_data; /**< User data */
	int count; /**< Number of pending req */
} ua_callback_s;

/**
 * @brief Monitor data structure.
 * @since_tizen 6.5
 */
typedef struct {
	unsigned int sensor_bitmask; /**< Bit-mask for sensors which be supported */
	unsigned int presence_detected_bitmask; /**< Detetected presence bitmask */
	unsigned int absence_detected_bitmask; /**< Detected Absence bitmask */

	unsigned int presence_bitmask_and; /**< AND operation bitmask for presence */
	unsigned int presence_bitmask_or; /**< OR operation bitmask for presence */
	ua_condition_conjunction_e presence_conjunction; /**< conjunction operator for presence*/
	unsigned int absence_bitmask_and; /**< AND operation bitmask for absence */
	unsigned int absence_bitmask_or; /**< OR operation bitmask for absence */
	ua_condition_conjunction_e absence_conjunction; /**< conjunction operator for absence*/

	ua_detection_mode_e presence_mode; /**< Presence mode */
	ua_detection_mode_e absence_mode; /**< Absence mode */
	gboolean presence_detection_started; /**< Is presence detection started */
	gboolean absence_detection_started; /**< Is absence detection started */
	gboolean scan_device_started; /**< Is scan device started */
	int internal_presence_started; /**< Is Internally scan device started */
	ua_callback_s absence_user_cb; /**< Callback to let apps know user is absence */
	ua_callback_s presence_user_cb; /**< Callback to let apps know user is presence */
	ua_scan_completed_cb scan_device_cb; /**< Callback to let apps know scanned registered devices */
	ua_presence_detected_cb presence_cb; /**< User presence detection callback */
	ua_absence_detected_cb absence_cb; /**< Absence detection callback */
	ua_callback_s sensor_status_cb; /**< Senser status changed callback */
	char *service; /**< Service name */
	GSList *user_state; /**< User current state i.e presence or absence */
	void *user_data; /**< User data */
	gboolean env_user_cb_sent; /** User presence callback sent or not flag for environmental sensors */

	unsigned int location_detected_bitmask; /**< Detected Location bitmask */
	gboolean location_detection_started; /**< Is location detection started */
	ua_location_detected_cb location_cb; /**< Location detection callback */
	unsigned int location_period; /**< Location callback period */
} ua_monitor_s;

/**
 * @brief sensor info data structure.
 * @since_tizen 6.5
 */
typedef struct {
	unsigned int status; /**< Sensor status 1:PRESENCE 2:ABSENCE */
	ua_sensor_h handle; /**< Sensor handle */
	ua_sensor_type_e bitmask; /**< Sensor bitmask */
	unsigned long long timestamp; /**< Timestamp */
	int accuracy; /**< Accuracy from sensor device */
	int count; /**< How many sensor data included */
	double *values; /**< Sensor data */
} ua_sensor_info_s;

/**
 * @brief location info data structure.
 * @since_tizen 6.5
 */
typedef struct {
	ua_monitor_h monitor_handle; /**< Monitor handle */
	ua_service_h service_handle; /**< Service handle */
	ua_monitor_h device_handle; /**< Device handle */
	ua_monitor_h user_handle; /**< User handle */
	ua_sensor_h sensor_handle; /**< Sensor handle */
	ua_sensor_type_e bitmask; /**< Sensor bitmask */
	unsigned int distance; /**< Device distance in mm */
	unsigned int x; /**< Device x coordinate */
	unsigned int y; /**< Device y coordinate */
	unsigned int z; /**< Device z coordinate */
} ua_location_info_s;

/**
 * @brief User state data structure.
 * @since_tizen 6.5
 */
typedef struct {
	char *account; /**< Account */
	unsigned int sensor_bitmask; /**< Detected sensor type bit mask */
	GSList *found_devices; /**< Detected devices' handles list */
	gboolean cb_sent; /** User presence callback sent or not flag */
} ua_user_state_info_s;

//TODO Unused struct.
/**
 * @brief Sensor state data structure.
 * @since_tizen 6.5
 */
typedef struct {
	ua_monitor_s *monitor; /**< Monitor handle */
	ua_sensor_type_e type; /**< Bitmask to express the supported sensors */
	unsigned short absence_period; /**< Absence period */
	unsigned int absence_timer; /**< Absence timer */
	unsigned int detection_timer; /**< Detection timer */
} ua_sensors_s;

/**
 * @brief User info. data structure.
 * @since_tizen 6.5
 */
typedef struct {
	ua_service_h service_handle; /**< Service handle */
	ua_user_h user_handle; /**< User handle */
	ua_presence_state_e state; /**< Presence state */
	unsigned int sensor_bitmask; /**< Detected user devices bitmask */
	char *account; /**< Account */
	char *name; /**< User name */
	gboolean isadded; /**< Is the user addition completed? */
	gboolean default_user; /**< Is it a default user? */
	gboolean create_by_app; /**< Did app add this user information? */
} ua_user_info_s;

/**
 * @brief Service info. data structure.
 * @since_tizen 6.5
 */
typedef struct {
	ua_service_h service_handle; /**< Service handle */
	char *name; /**< Service name */
	int presence_threshold;
	int absence_threshold;
	gboolean isadded; /**< Is the service addition completed? */
	gboolean default_service; /**< Is it a default service? */
	gboolean create_by_app; /**< Did app add this service information? */
} ua_service_info_s;

/**
 * @brief Detection type.
 * @since_tizen 6.5
 */
typedef enum {
	UA_PRESENCE_DETECTION = 0x00, /**< Presence detcection */
	UA_ABSENCE_DETECTION, /**< Absence detection */
	UA_LOCATION_DETECTION /**< Location detection */
} ua_detection_type_e;

/**
 * @brief BLE payload data structure, used for filter.
 * @since_tizen 6.5
 */
typedef struct {
	ua_payload_h handle; /**< Payload handle */
	unsigned char primary_key; /**< primary key for manufacturer data */
	unsigned char secondary_key; /**< secondary_key for manufacturer data */
	unsigned char device_type; /**< device type for manufacturer data */
	unsigned char extension_value1; /** extension value 1 for manufacturer data */
	unsigned char extension_value2; /** extension value 2 for manufacturer data */
	int device_uid_len; /** device_uid length for manufaturer data*/
	char *device_uid; /**< device_uid for manufacturer data */
	char *user_data; /**< user data for manufacturer data */
	gboolean isadded; /**< Is the payload addition completed? */
} ua_payload_info_s;

/**
 * @brief Device info data structure.
 * @since_tizen 6.5
 */
typedef struct {
	ua_user_h user; /**< User handle */
	ua_device_h handle; /**< Device handle */
	char *mac; /**< MAC address */
	char *bssid; /**< BSSID */
	char *ipv4; /**< IPv4 address */
	char *ipv6; /**< IPv6 address */
	char *device_id; /**< Mobile id (e.g. BT MAC or UUID) */
	ua_mac_type_e type; /**< Connectivity technology type using this MAC address */
	bool pairing_required; /**< Does it need pairing? */
	ua_presence_state_e state; /**< Presence state */
	ua_os_type_e os; /**< OS type */
	gboolean isadded; /**< Is the device addition completed? */
	gboolean create_by_app; /**< Did app add this device information? */
	gboolean discriminant; /**< Determines whether to judge PRESENCE/ABSENCE */
	unsigned long long last_seen; /**< The time of last presence */
	int rssi; /* RSSI value for device */
} ua_dev_info_s;

/**
 * @brief Callback to be invoked on absence detection for one user.
 * @since_tizen 6.5
 *
 * @remarks The @a handle should not be released.
 * @remarks The @a handle can be used only in the callback.
 * @remarks The @a user_handle should not be released.
 * @remarks The @a user_handle can be used only in the callback.
 * @remarks The @a sensor_handles should not be released.
 * @remarks The @a sensor_handles can be used only in the callback.
 *
 * @param[in] result The result of the requested operation.
 * @param[in] handle The monitor handle with which absence detection start was invoked.
 * @param[in] user_handle The user handle to detected user's information.
 *   In case monitor has more than one sensor and detection mode is not
 *   #UA_DETECT_MODE_ANY_SENSOR, sensor will be set to #UA_SENSOR_MAX before invoking
 *   callback.
 * @param[in] sensor_handles The list of sensor handles on which user is not detected.
 *
 * @exception
 * @pre
 * @post
 */
typedef void (*ua_absence_user_detected_cb)(
		int result,
		ua_monitor_h handle,
		ua_user_h user_handle,
		GSList *sensor_handles,
		void *user_data);

/**
 * @brief Callback to be invoked on presence detection for one user.
 * @since_tizen 6.5
 *
 * @remarks The @a handle should not be released.
 * @remarks The @a handle can be used only in the callback.
 * @remarks The @a user_handle should not be released.
 * @remarks The @a user_handle can be used only in the callback.
 * @remarks The @a device_handles should not be released.
 * @remarks The @a device_handles can be used only in the callback.
 * @remarks The @a sensor_handles should not be released.
 * @remarks The @a sensor_handles can be used only in the callback.
 *
 * @param[in] result The result of the requested operation.
 * @param[in] handle The monitor handle with which presence detection start was invoked.
 * @param[in] user_handle The user handle to detected user's information.
 * @param[in] device_handles The list of device handles on which user is detected.
 * @param[in] sensor_handles The list of sensor handles on which user is detected.
 *
 * @exception
 * @pre
 * @post
 */
typedef void (*ua_presence_user_detected_cb)(
		int result,
		ua_monitor_h handle,
		ua_user_h user_handle,
		GSList *device_handles,
		GSList *sensor_handles,
		void *user_data);

/**
 * @brief Callback to get all payload handles.
 * @since_tizen 6.5
 *
 * @remarks The @a payload_handle should not be released.
 * @remarks The @a payload_handle can be used only in the callback.
 *
 * @param[in] payload_handle The payload handle.
 * @param[in] user_data User data passed in ua_payload_foreach_added().
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_payload_foreach_added()
 */
typedef bool (*ua_added_payload_cb)(
		ua_payload_h payload_handle,
		void *user_data);

/**
 * @brief Calls the scan callback when the scan is complete.
 * @since_tizen 6.5
 *
 * @param[in] result Result code
 * @param[in] uam_info Device information
 *
 * @exception
 * @pre
 * @post
 */
void _ua_monitor_handle_scanned_device(int result, uam_device_info_s *uam_info);

/**
 * @brief Clears data related to scan in the monitor.
 * @since_tizen 6.5
 *
 * @param[in] result Result code.
 *
 * @exception
 * @pre
 * @post
 */
void _ua_monitor_handle_scan_complete(int result);

/**
 * @brief Calls the presence callback when detected registerd devices.
 * @since_tizen 6.5
 *
 * @param[in] sensor_info Sensor info for detected sensors.
 * @param[in] service Service information.
 * @param[in] account Account information.
 *
 * @exception
 * @pre
 * @post
 */
void _ua_monitor_handle_user_presence_detected(uam_sensor_info_s *sensor_info,
	char *service, char *account, unsigned long long timestamp, char *device_id);

/**
 * @brief Stops monitoring detection.
 * @since_tizen 6.5
 *
 * @param[in] svc_name Service name.
 * @param[in] cycle_state Detection cycle in mid or end.
 *
 * @exception
 * @pre
 * @post
 */
void _ua_monitor_handle_detection_stopped(char *svc_name,
	uam_cycle_state_e cycle_state);

/**
 * @brief Changes sensor state.
 * @since_tizen 6.5
 *
 * @param[in] bitmask Bitmask for sensors.
 * @param[in] ready Ready or not.
 *
 * @exception
 * @pre
 * @post
 */
void _ua_monitor_handle_sensor_state(unsigned int bitmask, gboolean ready);

/**
 * @brief Calls the absence callback when detected registerd devices.
 * @since_tizen 6.5
 *
 * @param[in] bitmask Bitmask for sensors
 * @param[in] service Service information
 * @param[in] account Account information
 *
 * @exception
 * @pre
 * @post
 */
void _ua_monitor_handle_user_absence_detected(
		uam_sensor_info_s *sensor_info, char *service, char *account);

/**
 * @brief Calls the location callback when detected registerd devices.
 * @since_tizen 6.5
 *
 * @param[in] bitmask Bitmask for sensors
 * @param[in] service Service information
 * @param[in] account Account information
 *
 * @exception
 * @pre
 * @post
 */
void _ua_monitor_handle_user_location_detected(
		uam_sensor_info_s *sensor_info, char *service,
		char *account, uam_device_info_s *device,
		char *device_id);

/**
 * @brief Calls the sensor status changed callback when notified of sensor status changed.
 * @since_tizen 6.5
 *
 * @param[in] info Sensor information
 *
 * @exception
 * @pre
 * @post
 */
void _ua_monitor_handle_sensor_status_changed(uam_sensor_info_s *info);

/**
 * @brief Checks whether ua is initialized or not.
 * @since_tizen 6.5
 *
 * @exception
 * @pre
 * @post
 */
bool _ua_is_initialized(void);

/**
 * @brief Destroys a monitor handle.
 * @since_tizen 6.5
 *
 * @param[in] data Monitor handle.
 *
 * @exception
 * @pre
 * @post
 */
void _ua_free_ua_monitor_t(gpointer data);

/**
 * @brief Clears monitor handle list.
 * @since_tizen 6.5
 *
 * @exception
 * @pre
 * @post
 */
void _ua_clear_monitor_list(void);

/**
 * @brief Gets a user handle.
 * @since_tizen 6.5
 *
 * @param[in] account User account.
 *
 * @exception
 * @pre
 * @post
 */
ua_user_info_s *_ua_get_user_from_list(const char *account);

/**
 * @brief Destroys a user handle.
 * @since_tizen 6.5
 *
 * @param[in] user User handle.
 *
 * @exception
 * @pre
 * @post
 */
void _ua_free_ua_user_info_t(gpointer data);

/**
 * @brief Clears user handle list.
 * @since_tizen 6.5
 *
 * @exception
 * @pre
 * @post
 */
void _ua_clear_user_list(void);

/**
 * @brief Destroys a service handle.
 * @since_tizen 6.5
 *
 * @param[in] service Service handle.
 *
 * @exception
 * @pre
 * @post
 */
void _ua_free_ua_service_info_s(gpointer data);

/**
 * @brief Clears service handle list.
 * @since_tizen 6.5
 *
 * @exception
 * @pre
 * @post
 */
void _ua_clear_service_list(void);

/**
 * @brief Add a device handle to internal device-list.
 * @since_tizen 6.5
 *
 * @param[in] dev_info Device handle.
 *
 * @exception
 * @pre
 * @post
 */
int _ua_device_add_info_to_list(ua_dev_info_s* dev_info);

/**
 * @brief Gets a user callback from internal callbacks.
 * @since_tizen 6.5
 *
 * @param[in] event User event.
 * @param[in] type MAC type.
 *
 * @exception
 * @pre
 * @post
 */
const void *_ua_get_user_callback(ua_user_event_e event, ua_mac_type_e type);

/**
 * @brief Adds a user callback to internal callbacks.
 * @since_tizen 6.5
 *
 * @param[in] event User event.
 * @param[in] type MAC type.
 * @param[in] callback User device callback.
 * @param[in] user_data User data.
 *
 * @exception
 * @pre
 * @post
 */
void _ua_add_user_callback(ua_user_event_e event, ua_mac_type_e type,
		ua_user_device_added_cb callback, void *user_data);

/**
 * @brief Gets a device handle.
 * @since_tizen 6.5
 *
 * @param[in] account User account.
 *
 * @exception
 * @pre
 * @post
 */
ua_dev_info_s *_ua_get_device_from_list(const char *device_id,
	const char *mac, ua_mac_type_e type);

/**
 * @brief Destroys a device handle.
 * @since_tizen 6.5
 *
 * @param[in] data Device handle.
 *
 * @exception
 * @pre
 * @post
 */
void _ua_free_ua_device_info_t(gpointer data);

/**
 * @brief Clears device handle list.
 * @since_tizen 6.5
 *
 * @exception
 * @pre
 * @post
 */
void _ua_clear_device_list(void);

/**
 * @brief Dereferences a user handle from device list.
 * @since_tizen 6.5
 *
 * @param[in] ua_info User info.
 *
 * @exception
 * @pre
 * @post
 */
void _ua_unref_user_from_device_list(ua_user_info_s *ua_info);

/**
 * @brief Gets a device handle.
 * @since_tizen 6.5
 *
 * @remarks The returned value should not be released.
 * @remarks The returned value is managed by the platform and will be released when destrying user list.
 *
 * @param[in] uam_info Device info. handle
 *
 * @return not null on success, otherwise error
 *
 * @exception
 * @pre
 * @post
 */
ua_dev_info_s* _ua_get_device_info_from_uam(uam_device_info_s *uam_info);

/**
 * @brief Sets data in user info. state callback.
 * @since_tizen 6.5
 *
 * @param[in] account Account.
 * @param[in] state #UA_PRESENCE_STATE_ACTIVE or #UA_PRESECE_STATE_INACTIVE.
 * @param[in] sensor_bitmask Sensor bitmask.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_NO_DATA No data
 *
 * @exception
 * @pre
 * @post
 */
int _ua_mark_user_info_state_db(char *account, int state, unsigned sensor_bitmask);

/**
 * @brief Checks whether user's detection type.
 * @since_tizen 6.5
 *
 * @param[in] bitmask Sensor bitmask.
 *
 * @return true on success, otherwise error
 * @retval true Successful
 * @retval false Failed
 *
 * @exception
 * @pre
 * @post
 */
bool _ua_check_all_users_absence_any(unsigned int bitmask);

/**
 * @brief Adds a user info to the list.
 * @since_tizen 6.5
 *
 * @param[in] ua_info User info.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 */
int _ua_user_add_info_to_list(ua_user_info_s* ua_info);

/**
 * @brief Adds a user info to the list in ua-manager.
 * @since_tizen 6.5
 *
 * @param[in] uam_info User info.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 */
int _ua_user_add_info_to_list_from_uapi_data(uam_user_info_s *uam_info);

/**
 * @brief Removes a user info to the list.
 * @since_tizen 6.5
 *
 * @param[in] account User account.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_NO_DATA No data
 *
 * @exception
 * @pre
 * @post
 */
int _ua_remove_user_info_from_list(char *account);

/**
 * @brief Adds a service info to the list.
 * @since_tizen 6.5
 *
 * @param[in] ua_info Service info.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 */
int _ua_service_add_info_to_list(ua_service_info_s* ua_info);

/**
 * @brief Adds a service info of initial type as uam_service_info_s to the list.
 * @since_tizen 6.5
 *
 * @param[in] uam_info Service info.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 */
int _ua_service_add_info_to_list_from_uam_data(uam_service_info_s *uam_info);

/**
 * @brief Gets a service info as uam_service_info_s from the the list.
 * @since_tizen 6.5
 *
 * @param[in] service_name Service name.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 */
ua_service_info_s* __ua_get_service_from_list(const char *service_name);

/**
 * @brief Removes a service info from the list.
 * @since_tizen 6.5
 *
 * @param[in] uam_info Service info.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_NO_DATA No data
 *
 * @exception
 * @pre
 * @post
 */
int _ua_service_remove_info_from_list(uam_service_info_s *uam_info);

/**
 * @brief Adds a device info. to DB in ua-manager.
 * @since_tizen 6.5
 *
 * @param[in] result Result code.
 * @param[in] uam_info Device information.
 *
 * @exception
 * @pre
 * @post
 */
void _ua_handle_device_added(int result, uam_device_info_s *uam_info);

/**
 * @brief Removes a device info. from DB in ua-manager.
 * @since_tizen 6.5
 *
 * @param[in] result Result code.
 * @param[in] uam_info Device information.
 *
 * @exception
 * @pre
 * @post
 */
void _ua_handle_device_removed(int result, uam_device_info_s *uam_info);

/**
 * @brief Retrieves the device handle of all the registered devices for one specific user.
 * @since_tizen 6.5
 *
 * @param[in] user_handle The user handle.
 * @param[in] foreach_cb Callback function to be invoked with user device handle.
 * @param[in] user_data The user data to be passed when callback is called.
 *
 * @exception
 * @pre
 * @post
 */
int _ua_foreach_devices_by_user(
		ua_user_h user_handle,
		ua_registered_device_cb foreach_cb,
		void *user_data);

/**
 * @brief Callback to get the service handle.
 * @since_tizen 6.5
 *
 * @remarks The @a service_handle should not be released.
 * @remarks The @a service_handle can be used only in the callback.
 *
 * @param[in] service_handle The service handle.
 * @param[in] user_data User data passed in _ua_intr_foreach_registered_services().
 *
 * @exception
 * @pre
 * @post
 *
 * @see _ua_intr_foreach_registered_services()
 */
typedef bool (*_ua_intr_registered_service_cb)(ua_service_h service_handle, void *user_data);

/**
 * @brief Retrieves the service handle of all the registered services in ua-manager.
 * @since_tizen 6.5
 *
 * @param[in] foreach_cb Callback function to be invoked with service handle.
 * @param[in] user_data The user data to be passed when callback is called.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see _ua_intr_registered_service_cb()
 */
int _ua_intr_foreach_registered_services(_ua_intr_registered_service_cb foreach_cb,
		void *user_data);

/**
 * @brief Retrieves the service handle of all the registered services.
 * @since_tizen 6.5
 *
 * @param[in] foreach_cb Callback function to be invoked with service handle.
 * @param[in] user_data The user data to be passed when callback is called.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_service_cb()
 */
int _ua_foreach_registered_services(ua_service_cb foreach_cb,
		void *user_data);

/**
 * @brief Gets default service data.
 * @since_tizen 6.5
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 */
int _ua_intr_get_default_service(void);

/**
 * @brief Callback to get the user handle added to the service.
 * @since_tizen 6.5
 *
 * @remarks The @a user_handle should not be released.
 * @remarks The @a user_handle can be used only in the callback.
 *
 * @param[in] user_handle The user handle.
 * @param[in] user_data User data passed in _ua_intr_foreach_registered_users().
 *
 * @exception
 * @pre
 * @post
 */
typedef bool (*_ua_intr_registered_user_cb)(ua_user_h user_handle, void *user_data);

/**
 * @brief Retrieves the user handle of all the registered users in ua-manager.
 * @since_tizen 6.5
 *
 * @param[in] foreach_cb Callback function to be invoked with user handle.
 * @param[in] user_data The user data to be passed when callback is called.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see _ua_intr_registered_user_cb()
 */
int _ua_intr_foreach_registered_users(_ua_intr_registered_user_cb foreach_cb,
		void *user_data);

/**
 * @brief Retrieves the user handle of all the registered users.
 * @since_tizen 6.5
 *
 * @param[in] foreach_cb Callback function to be invoked with user handle.
 * @param[in] user_data The user data to be passed when callback is called.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 * @see ua_registered_user_cb()
 */
int _ua_foreach_registered_users(ua_registered_user_cb foreach_cb,
		void *user_data);

/**
 * @brief Gets default user data.
 * @since_tizen 6.5
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 */
int _ua_intr_get_default_user(ua_user_h * user_handle);

/**
 * @brief Gets a user handle by the account.
 * @since_tizen 6.5
 *
 * @remarks The returned value should not be released.
 * @remarks The returned value is managed by the platform and will be released when destrying user list.
 *
 * @param[in] account Account
 *
 * @return not null on success, otherwise error
 *
 * @exception
 * @pre
 * @post
 */
ua_user_h _ua_get_user_handle_by_account(const char *account);

/**
 * @brief Checks whether a device exists in device list.
 * @since_tizen 6.5
 *
 * @param[in] device_id The device id.
 * @param[in] mac The device MAC address.
 * @param[in] type The device type.
 * @param[in] is_exist Whether exists or not.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 */
int _ua_is_device_exist(char *device_id, char *mac, ua_mac_type_e type,
		gboolean *is_exist);

/**
 * @brief Gets the service list.
 * @since_tizen 6.5
 *
 * @remarks The returned value should not be released.
 * @remarks The returned value is managed by the platform and will be released when destroying service list.
 *
 * @return not null on success, otherwise error
 *
 * @exception
 * @pre
 * @post
 */
GSList *_ua_service_get_services(void);

/**
 * @brief Gets the user list.
 * @since_tizen 6.5
 *
 * @remarks The returned value should not be released.
 * @remarks The returned value is managed by the platform and will be released when destroying user list.
 *
 * @return not null on success, otherwise error
 *
 * @exception
 * @pre
 * @post
 */
GSList *_ua_user_get_users(void);

#ifdef __cplusplus
}
#endif
#endif /* __TIZEN_CAPI_NETWORK_USER_AWARENESS_PRIVATE_H__ */
