/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __TIZEN_CAPI_NETWORK_USER_AWARENESS_UTIL_H__
#define __TIZEN_CAPI_NETWORK_USER_AWARENESS_UTIL_H__

#include <dlog.h>
#include <ua-api.h>
#include <user-awareness.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @internal
 * @brief Returns changed technology type for ua-manager.
 * @since_tizen 6.5
 *
 * @param[in] ua_error Error code to be converted.
 *
 * @return #UA_MAC_TYPE_INVALID on failure, otherwise a positive error value
 * @retval #UA_MAC_TYPE_NONE None
 * @retval #UA_MAC_TYPE_BT BT MAC
 * @retval #UA_MAC_TYPE_BLE BLE MAC
 * @retval #UA_MAC_TYPE_WIFIWi-Fi station mode MAC
 * @retval #UA_MAC_TYPE_P2P Wi-Fi P2P mode MAC
 *
 * @exception
 * @pre
 * @post
 *
 */
ua_mac_type_e _to_ua_mac_type(uam_tech_type_e tech_type);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @internal
 * @brief Converts the error code of ua-manager to the error code of user-awareness.
 * @since_tizen 6.5
 *
 * @param[in] ua_error Error code to be converted.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_NOT_SUPPORTED Not supported
 * @retval #UA_ERROR_PERMISSION_DENIED Permission denied
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_NOT_PERMITTED Operation not permitted
 * @retval #UA_ERROR_TIMED_OUT Timeout
 * @retval #UA_ERROR_NOT_INITIALIZED Not initialized
 * @retval #UA_ERROR_NOT_IN_PROGRESS Not in progress
 * @retval #UA_ERROR_NOW_IN_PROGRESS Now in progress
 * @retval #UA_ERROR_ALREADY_DONE Already done
 * @retval #UA_ERROR_NOT_FOUND Not found
 * @retval #UA_ERROR_ALREADY_REGISTERED Already registered
 * @retval #UA_ERROR_DB_FAILED DB failed
 * @retval #UA_ERROR_NOT_REGISTERED Not registered
 * @retval #UA_ERROR_OPERATION_FAILED Operatoin failed
 *
 * @exception
 * @pre
 * @post
 *
 */
int _ua_get_error_code(int ua_error);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @internal
 * @brief Converts the error code to a string.
 * @since_tizen 6.5
 *
 * @remarks The returned value can be used until return call function is valid.
 *
 * @param[in] err Error code to be converted.
 *
 * @return Error string
 *
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_NOT_SUPPORTED Not Supported
 * @retval #UA_ERROR_PERMISSION_DENIED Permission denied
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_NOT_PERMITTED Operation not permitted
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 * @retval #UA_ERROR_RESOURCE_BUSY Device or resource busy
 * @retval #UA_ERROR_TIMED_OUT Timeout error
 * @retval #UA_ERROR_CANCELED Operation canceled
 * @retval #UA_ERROR_NOW_IN_PROGRESS Operation now in progress
 * @retval #UA_ERROR_NO_DATA No data available
 * @retval #UA_ERROR_NOT_INITIALIZED Not initialized
 * @retval #UA_ERROR_NOT_IN_PROGRESS Operation Not in progress
 * @retval #UA_ERROR_ALREADY_DONE Operation already done
 * @retval #UA_ERROR_OPERATION_FAILED Operation Failed
 * @retval #UA_ERROR_NOT_READY Resource not ready
 * @retval #UA_ERROR_NOT_ENABLED Not enabled
 * @retval #UA_ERROR_NOT_FOUND Not found
 * @retval #UA_ERROR_ALREADY_REGISTERED Already registered
 * @retval #UA_ERROR_DB_FAILED DB operation faield
 * @retval #UA_ERROR_NOT_REGISTERED Not initialized
 *
 * @exception
 * @pre
 * @post
 *
 */
const char* _ua_get_error_string(ua_error_e err);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @internal
 * @brief Converts the tect. type used in the client-library to the type used by the ua-manager.
 * @since_tizen 6.5
 *
 * @remarks The returned value can be used until return call function is valid.
 *
 * @param[in] type Tech code to be converted.
 *
 * @return technology type
 *
 * @retval #UA_TECH_TYPE_BT Bluetooth
 * @retval #UA_TECH_TYPE_BLE Bluetooth Low Energy
 * @retval #UA_TECH_TYPE_WIFI Wi-Fi
 * @retval #UA_TECH_TYPE_P2P Wi-Fi Direct
 * @retval #UA_MAC_TYPE_INVALID Invalid
 *
 * @exception
 * @pre
 * @post
 *
 */
uam_tech_type_e _ua_to_uam_tech_type(ua_mac_type_e type);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @internal
 * @brief Converts device's tech. type enum to sensor bitmask enum.
 * @since_tizen 6.5
 *
 * @remarks The returned value can be used until return call function is valid.
 *
 * @param[in] type Tech code to be converted.
 *
 * @return Sensor bitmask
 *
 * @retval #UA_SENSOR_BT Bluetooth
 * @retval #UA_SENSOR_BLE Bluetooth Low Energy
 * @retval #UA_SENSOR_WIFI Wi-Fi
 * @retval #UA_SENSOR_WIFI_LOCATION Wi-Fi Location
 * @retval #UA_SENSOR_UWB UWB
 * @retval #UA_SENSOR_MAX Invalid
 *
 * @exception
 * @pre
 * @post
 *
 */
ua_sensor_type_e _ua_dev_type_to_sensor(ua_mac_type_e type);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @internal
 * @brief Converts sensor bitmask enum to device tech type enum.
 * @since_tizen 6.5
 *
 * @remarks The returned value can be used until return call function is valid.
 *
 * @param[in] type sensor code to be converted.
 *
 * @return Mac type
 *
 * @exception
 * @pre
 * @post
 *
 */
ua_mac_type_e _ua_sensor_to_dev_type(ua_sensor_type_e type);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @internal
 * @brief Converts uam sensor bitmask to ua sensor bitmask enum.
 * @since_tizen 6.5
 *
 * @remarks The returned value can be used until return call function is valid.
 *
 * @param[in] bitmask Uam sensor bitmask to be converted.
 *
 * @return Sensor bitmask
 *
 * @retval #UA_SENSOR_BT Bluetooth
 * @retval #UA_SENSOR_BLE Bluetooth Low Energy
 * @retval #UA_SENSOR_WIFI Wi-Fi
 * @retval #UA_SENSOR_MOTION Motion
 * @retval #UA_SENSOR_LIGHT Light
 * @retval #UA_SENSOR_AUDIO Audiio
 * @retval #UA_SENSOR_MAX Invalid
 *
 * @exception
 * @pre
 * @post
 *
 */
ua_sensor_type_e _uam_to_ua_sensor(uam_sensor_bitmask_e bitmask);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @internal
 * @brief Converts uam sensor info to ua sensor info.
 * @since_tizen 6.5
 *
 * @remarks The returned value can be used until return call function is valid.
 *
 * @param[in] Struct uam sensor info to be converted.
 *
 * @return Sensor info struct
 *
 * @exception
 * @pre
 * @post
 *
 */
ua_sensor_info_s* _uam_to_ua_sensor_info(uam_sensor_info_s *info);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @internal
 * @brief Converts uam location info to ua location info.
 * @since_tizen 6.5
 *
 * @remarks The returned value can be used until return call function is valid.
 *
 * @param[in] Struct uam device info to be converted.
 *
 * @return Location info struct
 *
 * @exception
 * @pre
 * @post
 *
 */
ua_location_info_s* _uam_to_ua_location_info(uam_device_info_s *info);


/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets sensor handle by sensor info.
 * @since_tizen 6.5
 *
 * @remarks The @a sensor_handle should not be released.
 * @remarks The @a sensor_handle can be used only in the function.
 *
 * @param[in] sensor_info The detected sensor information
 * @param[out] sensor_handle The sensor handle.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 *
 */
int _ua_sensor_get_by_sensor_info(
		ua_sensor_info_s *sensor_info,
		ua_sensor_h * sensor_handle);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Gets sensor handle by sensor info.
 * @since_tizen 6.5
 *
 * @remarks The @a sensor_handle should not be released.
 * @remarks The @a sensor_handle can be used only in the function.
 *
 * @param[in] sensor_bitmask The detected sensors bitmask
 * @param[out] sensor_handle The sensor handles list
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #UA_ERROR_OUT_OF_MEMORY Out of memory
 *
 * @exception
 * @pre
 * @post
 *
 */
GSList *_ua_sensor_get_sensor_handle_list(
		unsigned int sensor_bitmask);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Releases sensor info.
 * @since_tizen 6.5
 *
 * @param[in] sensor_info The sensor information to be freed.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 */
int _ua_free_sensor_info(ua_sensor_info_s *sensor_info);

/**
 * @brief Clears sensor handle list.
 * @since_tizen 6.5
 *
 * @exception
 * @pre
 * @post
 */
void _ua_clear_sensor_list(void);

/**
 * @brief Clears location handle list.
 * @since_tizen 6.5
 *
 * @exception
 * @pre
 * @post
 */
void _ua_clear_location_list(void);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Updates location info.
 * @since_tizen 6.5
 *
 * @param[in] location_info The location information to be updated.
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 * @retval #UA_ERROR_INVALID_PARAMETER Invalid parameter
 *
 * @exception
 * @pre
 * @post
 *
 */
int _ua_update_location_info(ua_location_info_s *location_info);

/**
 * @ingroup CAPI_NETWORK_UA_MODULE
 * @brief Sends location callback.
 * @since_tizen 6.5
 *
 * @return 0 on success, otherwise a negative error value
 * @retval #UA_ERROR_NONE Successful
 *
 * @exception
 * @pre
 * @post
 *
 */
gboolean _ua_monitor_send_location_cb(gpointer data);

#ifdef __cplusplus
}
#endif
#endif /* __TIZEN_CAPI_NETWORK_USER_AWARENESS_UTIL_H__ */
