/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <glib.h>
#include <ua-api.h>

#include <user-awareness.h>
#include <user-awareness-log.h>
#include <user-awareness-internal.h>
#include <user-awareness-private.h>

static GSList *ua_sensors_list = NULL; /* ua_sensor_info_s */
static GSList *ua_location_list = NULL; /* ua_location_info_s */

static ua_feature_table_t feature_table[] = {
	{"tizen.org/feature/user_awareness", FALSE, FALSE}, /* UA_FEATURE_COMMON */
	{"tizen.org/feature/user_awareness.location", FALSE, FALSE}, /* UA_FEATURE_LOCATION */
};

int _ua_check_supported_feature(ua_feature_t feature, bool *supported)
{
	int ret;

	if (feature < UA_FEATURE_BASE || feature >= UA_FEATURE_MAX) {
		UA_ERR("Invalid parameter. feature [%d]", feature);
		return UA_ERROR_INVALID_PARAMETER;
	}

	if (TRUE == feature_table[feature].is_check) {
		*supported = feature_table[feature].value;
		return UA_ERROR_NONE;
	}

	ret = system_info_get_platform_bool(feature_table[feature].name, supported);
	if (SYSTEM_INFO_ERROR_NONE != ret) {
		UA_ERR("fail to get %s", feature_table[feature].name);
		return UA_ERROR_OPERATION_FAILED;
	}

	feature_table[feature].is_check = TRUE;
	feature_table[feature].value = *supported;

	return UA_ERROR_NONE;
}

ua_mac_type_e _to_ua_mac_type(int tech_type)
{
	switch (tech_type) {
	case UAM_TECH_TYPE_BT:
		return UA_MAC_TYPE_BT;
	case UAM_TECH_TYPE_BLE:
		return UA_MAC_TYPE_BLE;
	case UAM_TECH_TYPE_WIFI:
		return UA_MAC_TYPE_WIFI;
	case UAM_TECH_TYPE_P2P:
		return UA_MAC_TYPE_P2P;
	case UAM_TECH_TYPE_WIFI_LOCATION:
		return UA_MAC_TYPE_WIFI_LOCATION;
	case UAM_TECH_TYPE_UWB:
		return UA_MAC_TYPE_UWB;
	default:
		return UA_MAC_TYPE_INVALID;
	}
}

int _ua_get_error_code(int ua_error)
{
	switch (ua_error) {
	case UAM_ERROR_NONE:
		return UA_ERROR_NONE;
	case UAM_ERROR_INVALID_PARAMETER:
		return UA_ERROR_INVALID_PARAMETER;
	/* LCOV_EXCL_START */
	case UAM_ERROR_RESOURCE_BUSY:
		return UA_ERROR_RESOURCE_BUSY;
	case UAM_ERROR_PERMISSION_DENIED:
		return UA_ERROR_PERMISSION_DENIED;
	case UAM_ERROR_TIMED_OUT:
		return UA_ERROR_TIMED_OUT;
	case UAM_ERROR_NOW_IN_PROGRESS:
		return UA_ERROR_NOW_IN_PROGRESS;
	case UAM_ERROR_NOT_SUPPORTED:
		return UA_ERROR_NOT_SUPPORTED;
	case UAM_ERROR_NOT_INITIALIZED:
		return UA_ERROR_NOT_INITIALIZED;
	case UAM_ERROR_NOT_IN_OPERATION:
		return UA_ERROR_NOT_IN_PROGRESS;
	case UAM_ERROR_ALREADY_DONE:
		return UA_ERROR_ALREADY_DONE;
	case UAM_ERROR_INTERNAL:
		return UA_ERROR_OPERATION_FAILED;
	/* LCOV_EXCL_STOP */
	case UAM_ERROR_NOT_FOUND:
		return UA_ERROR_NOT_FOUND;
	/* LCOV_EXCL_START */
	case UAM_ERROR_ALREADY_REGISTERED:
		return UA_ERROR_ALREADY_REGISTERED;
	case UAM_ERROR_DB_FAILED:
		return UA_ERROR_DB_FAILED;
	case UAM_ERROR_NOT_REGISTERED:
		return UA_ERROR_NOT_REGISTERED;
	/* LCOV_EXCL_STOP */
	default:
		return UA_ERROR_OPERATION_FAILED;
	}
}

const char *_ua_get_error_string(ua_error_e err)
{
	const char *err_str = NULL;

	switch (err) {
	case UA_ERROR_NONE:
		err_str = "UA_ERROR_NONE";
		break;
	/* LCOV_EXCL_START */
	case UA_ERROR_NOT_PERMITTED:
		err_str = "UA_ERROR_NOT_PERMITTED";
		break;
	/* LCOV_EXCL_STOP */
	case UA_ERROR_INVALID_PARAMETER:
		err_str = "UA_ERROR_INVALID_PARAMETER";
		break;
	/* LCOV_EXCL_START */
	case UA_ERROR_OUT_OF_MEMORY:
		err_str = "UA_ERROR_OUT_OF_MEMORY";
		break;
	case UA_ERROR_PERMISSION_DENIED:
		err_str = "UA_ERROR_PERMISSION_DENIED";
		break;
	case UA_ERROR_RESOURCE_BUSY:
		err_str = "UA_ERROR_RESOURCE_BUSY";
		break;
	case UA_ERROR_TIMED_OUT:
		err_str = "UA_ERROR_TIMED_OUT";
		break;
	case UA_ERROR_CANCELED:
		err_str = "UA_ERROR_CANCELED";
		break;
	case UA_ERROR_NOW_IN_PROGRESS:
		err_str = "UA_ERROR_NOW_IN_PROGRESS";
		break;
	case UA_ERROR_NOT_SUPPORTED:
		err_str = "UA_ERROR_NOT_SUPPORTED";
		break;
	case UA_ERROR_NO_DATA:
		err_str = "UA_ERROR_NO_DATA";
		break;
	case UA_ERROR_NOT_INITIALIZED:
		err_str = "UA_ERROR_NOT_INITIALIZED";
		break;
	case UA_ERROR_NOT_IN_PROGRESS:
		err_str = "UA_ERROR_NOT_IN_PROGRESS";
		break;
	case UA_ERROR_ALREADY_DONE:
		err_str = "UA_ERROR_ALREADY_DONE";
		break;
	/* LCOV_EXCL_STOP */
	case UA_ERROR_OPERATION_FAILED:
		err_str = "UA_ERROR_OPERATION_FAILED";
		break;
	case UA_ERROR_NOT_FOUND:
		err_str = "UA_ERROR_NOT_FOUND";
		break;
	/* LCOV_EXCL_START */
	case UA_ERROR_ALREADY_REGISTERED:
		err_str = "UA_ERROR_ALREADY_REGISTERED";
		break;
	case UA_ERROR_DB_FAILED:
		err_str = "UA_ERROR_DB_FAILED";
		break;
	case UA_ERROR_NOT_REGISTERED:
		err_str = "UA_ERROR_NOT_REGISTERED";
		break;
	default:
		err_str = "NOT Defined";
		break;
	/* LCOV_EXCL_STOP */
	}

	return err_str;
}

uam_tech_type_e _ua_to_uam_tech_type(int type)
{
	switch (type) {
	case UA_MAC_TYPE_BT:
		return UAM_TECH_TYPE_BT;
	case UA_MAC_TYPE_BLE:
		return UAM_TECH_TYPE_BLE;
	case UA_MAC_TYPE_WIFI:
		return UAM_TECH_TYPE_WIFI;
	case UA_MAC_TYPE_P2P:
		return UAM_TECH_TYPE_P2P;
	case UA_MAC_TYPE_WIFI_LOCATION:
		return UAM_TECH_TYPE_WIFI_LOCATION;
	case UA_MAC_TYPE_UWB:
		return UAM_TECH_TYPE_UWB;
	default:
		return UA_MAC_TYPE_INVALID;
	}
}

ua_sensor_type_e _ua_dev_type_to_sensor(int type)
{
	FUNC_ENTRY;

	switch (type) {
	case UA_MAC_TYPE_BT:
		return UA_SENSOR_BT;
	case UA_MAC_TYPE_BLE:
		return UA_SENSOR_BLE;
	case UA_MAC_TYPE_WIFI:
		return UA_SENSOR_WIFI;
	case UA_MAC_TYPE_WIFI_LOCATION:
		return UA_SENSOR_WIFI_LOCATION;
	case UA_MAC_TYPE_UWB:
		return UA_SENSOR_UWB;
	default:
		UA_WARN("Uncompatible to convert. Mac type [%u]", type);
		return UA_SENSOR_MAX;
	}
	FUNC_EXIT;
}

ua_mac_type_e _ua_sensor_to_dev_type(int type)
{
	FUNC_ENTRY;

	switch (type) {
	case UA_SENSOR_BT:
		return UA_MAC_TYPE_BT;
	case UA_SENSOR_BLE:
		return UA_MAC_TYPE_BLE;
	case UA_SENSOR_WIFI:
		return UA_MAC_TYPE_WIFI;
	case UA_SENSOR_WIFI_LOCATION:
		return UA_MAC_TYPE_WIFI_LOCATION;
	case UA_SENSOR_UWB:
		return UA_MAC_TYPE_UWB;
	default:
		UA_WARN("Uncompatible to convert. sensor [%u]", type);
		return UA_MAC_TYPE_INVALID;
	}
	FUNC_EXIT;
}

ua_sensor_type_e _uam_to_ua_sensor(int bitmask)
{
	FUNC_ENTRY;

	switch (bitmask) {
	case UAM_SENSOR_BITMASK_BT:
		return UA_SENSOR_BT;
	case UAM_SENSOR_BITMASK_BLE:
		return UA_SENSOR_BLE;
	case UAM_SENSOR_BITMASK_WIFI:
		return UA_SENSOR_WIFI;
	case UAM_SENSOR_BITMASK_MOTION:
		return UA_SENSOR_MOTION;
	case UAM_SENSOR_BITMASK_LIGHT:
		return UA_SENSOR_LIGHT;
	case UAM_SENSOR_BITMASK_AUDIO:
		return UA_SENSOR_AUDIO;
	case UAM_SENSOR_BITMASK_WIFI_LOCATION:
		return UA_SENSOR_WIFI_LOCATION;
	case UAM_SENSOR_BITMASK_UWB:
		return UA_SENSOR_UWB;
	default:
		UA_WARN("Unknown sensor bitmask [0x%8.8X]", bitmask);
		return UA_SENSOR_MAX;
	}
	FUNC_EXIT;
}

static void __ua_free_ua_sensor_info_s(gpointer data)
{
	FUNC_ENTRY;
	ua_sensor_info_s *sensor_info = (ua_sensor_info_s*)data;

	if (sensor_info == NULL)
		return;
	g_free(sensor_info->values);
	g_free(sensor_info);
	FUNC_EXIT;
}

void _ua_clear_sensor_list(void)
{
	g_slist_free_full(ua_sensors_list, __ua_free_ua_sensor_info_s);
	ua_sensors_list = NULL;
}

static ua_sensor_info_s* __ua_allocate_ua_sensor_info()
{
	ua_sensor_info_s *sensor_info;
	sensor_info = g_malloc0(sizeof(ua_sensor_info_s));
	if (!sensor_info) {
		UA_ERR("g_malloc0 failed");
		return NULL;
	}
	sensor_info->values = NULL;
	sensor_info->values = g_malloc0(UA_SENSOR_MAX_VALUES*sizeof(double));
	if (!sensor_info->values) {
		UA_ERR("g_malloc0 failed");
		g_free(sensor_info);
		return NULL;
	}
	return sensor_info;
}

static ua_sensor_info_s *__ua_get_sensor_info_from_list(ua_sensor_type_e bitmask)
{
	GSList *l;
	ua_sensor_info_s *sensor;

	for (l = ua_sensors_list; NULL != l; l = g_slist_next(l)) {
		sensor = (ua_sensor_info_s *)l->data;

		if (bitmask == sensor->bitmask) {
			UA_INFO("Sensor info found, sensor bitmask[%d]", sensor->bitmask);
			return sensor;
		}
	}

	return NULL;
}

static int __ua_update_sensor_info(
		ua_sensor_info_s *sensor,
		ua_sensor_info_s *info)
{
	FUNC_ENTRY;
	unsigned int i = 0;

	UA_VALIDATE_INPUT_PARAMETER(sensor);
	UA_VALIDATE_INPUT_PARAMETER(info);

	sensor->timestamp = info->timestamp;
	sensor->count = info->count;
	sensor->bitmask = info->bitmask;
	for (i = 0; i < UA_SENSOR_MAX_VALUES; i++)
		sensor->values[i] = info->values[i];

	FUNC_EXIT;
	return UA_ERROR_NONE;
}

int _ua_free_sensor_info(ua_sensor_info_s *sensor_info)
{
	retv_if(NULL == sensor_info, UA_ERROR_INVALID_PARAMETER);
	g_free(sensor_info->values);
	g_free(sensor_info);

	return UA_ERROR_NONE;
}

ua_sensor_info_s* _uam_to_ua_sensor_info(uam_sensor_info_s *info)
{
	FUNC_ENTRY;
	ua_sensor_info_s *sensor_info = NULL;
	unsigned int i = 0;

	sensor_info = __ua_allocate_ua_sensor_info();
	if (!sensor_info)
		return NULL;

	sensor_info->status = info->status;
	sensor_info->timestamp = info->timestamp;
	sensor_info->count = info->count;
	sensor_info->bitmask = _uam_to_ua_sensor(info->sensor_bitmask);
	for (i = 0; i < UA_SENSOR_MAX_VALUES; i++)
		sensor_info->values[i] = info->values[i];

	return sensor_info;
	FUNC_EXIT;
}

int _ua_sensor_get_by_sensor_info(
		ua_sensor_info_s *info,
		ua_sensor_h *sensor_handle)
{
	FUNC_ENTRY;
	int ret = UA_ERROR_NONE;
	ua_sensor_info_s *sensor;
	*sensor_handle = NULL;

	UA_VALIDATE_INPUT_PARAMETER(info);
	UA_VALIDATE_INPUT_PARAMETER(sensor_handle);

	sensor = __ua_get_sensor_info_from_list(info->bitmask);

	if (NULL != sensor) {
		ret = __ua_update_sensor_info(sensor, info);
		*sensor_handle = (ua_sensor_h)sensor;
		goto done;
	}

	sensor = __ua_allocate_ua_sensor_info();
	if (!sensor)
		return UA_ERROR_OUT_OF_MEMORY;

	ret = __ua_update_sensor_info(sensor, info);

	/* Add sensor to list of sensors */
	*sensor_handle = (ua_sensor_h)sensor;
	sensor->handle = *sensor_handle;
	ua_sensors_list = g_slist_append(ua_sensors_list, sensor);

done:
	FUNC_EXIT;
	return ret;
}

GSList *_ua_sensor_get_sensor_handle_list(
		unsigned int bitmask)
{
	FUNC_ENTRY;
	GSList *sensors = NULL;
	GSList *l;
	ua_sensor_info_s *sensor;

	for (l = ua_sensors_list; NULL != l; l = g_slist_next(l)) {
		sensor = (ua_sensor_info_s *)l->data;

		if (bitmask & sensor->bitmask) {
			UA_INFO("Sensor info found, sensor bitmask[%d]", sensor->bitmask);
			sensors = g_slist_append(sensors, sensor->handle);
		}
	}

	FUNC_EXIT;
	return sensors;
}

int ua_sensor_get_status(
		ua_sensor_h sensor_handle,
		ua_sensor_report_e *status)
{
	FUNC_ENTRY;
	ua_sensor_info_s *sensor;

	UA_CHECK_SUPPORTED_FEATURE(UA_FEATURE_COMMON);
	UA_CHECK_INIT_STATUS();
	UA_VALIDATE_INPUT_PARAMETER(sensor_handle);
	UA_INFO("Sensor Handle [%p]", sensor_handle);
	sensor = (ua_sensor_info_s *)sensor_handle;

	*status = sensor->status;

	FUNC_EXIT;
	return UA_ERROR_NONE;
}

int ua_sensor_get_timestamp(
		ua_sensor_h sensor_handle,
		unsigned long long *timestamp)
{
	FUNC_ENTRY;
	ua_sensor_info_s *sensor;

	UA_CHECK_SUPPORTED_FEATURE(UA_FEATURE_COMMON);
	UA_CHECK_INIT_STATUS();
	UA_VALIDATE_INPUT_PARAMETER(sensor_handle);
	UA_INFO("Sensor Handle [%p]", sensor_handle);
	sensor = (ua_sensor_info_s *)sensor_handle;

	*timestamp = sensor->timestamp;

	FUNC_EXIT;
	return UA_ERROR_NONE;
}

int ua_sensor_get_type(
		ua_sensor_h sensor_handle,
		ua_sensor_type_e *type)
{
	FUNC_ENTRY;
	ua_sensor_info_s *sensor;

	UA_CHECK_SUPPORTED_FEATURE(UA_FEATURE_COMMON);
	UA_CHECK_INIT_STATUS();
	UA_VALIDATE_INPUT_PARAMETER(sensor_handle);
	UA_INFO("Sensor Handle [%p]", sensor_handle);
	sensor = (ua_sensor_info_s *)sensor_handle;

	*type = sensor->bitmask;

	FUNC_EXIT;
	return UA_ERROR_NONE;
}

static ua_location_info_s* __ua_allocate_ua_location_info()
{
	ua_location_info_s *location_info;
	location_info = g_malloc0(sizeof(ua_location_info_s));
	if (!location_info) {
		UA_ERR("g_malloc0 failed");
		return NULL;
	}
	return location_info;
}

ua_location_info_s* _uam_to_ua_location_info(uam_device_info_s *info)
{
	FUNC_ENTRY;
	ua_location_info_s *location_info = NULL;

	location_info = __ua_allocate_ua_location_info();
	if (!location_info)
		return NULL;

	location_info->distance = info->distance_mm;
	location_info->x = info->x;
	location_info->y = info->y;
	location_info->z = info->z;

	return location_info;
}

static ua_location_info_s *__ua_get_location_info_from_list(
				ua_service_h svc_h, ua_device_h dev_h)
{
	GSList *l;
	ua_location_info_s *location;

	for (l = ua_location_list; NULL != l; l = g_slist_next(l)) {
		location = (ua_location_info_s *)l->data;

		if (svc_h == location->service_handle &&
				dev_h == location->device_handle) {
			UA_INFO("Location info found for service and device");
			return location;
		}
	}

	return NULL;
}

static int __ua_update_location_info(
		ua_location_info_s *location,
		ua_location_info_s *info)
{
	FUNC_ENTRY;

	UA_VALIDATE_INPUT_PARAMETER(location);
	UA_VALIDATE_INPUT_PARAMETER(info);

	location->distance = info->distance;
	location->x = info->x;
	location->y = info->y;
	location->z = info->z;
	location->bitmask = info->bitmask;
	location->service_handle = info->service_handle;
	location->device_handle = info->device_handle;
	location->user_handle = info->user_handle;
	location->monitor_handle = info->monitor_handle;
	location->sensor_handle = info->sensor_handle;

	FUNC_EXIT;
	return UA_ERROR_NONE;
}

int _ua_update_location_info(
		ua_location_info_s *info)
{
	FUNC_ENTRY;
	int ret = UA_ERROR_NONE;
	ua_location_info_s *location;

	UA_VALIDATE_INPUT_PARAMETER(info);

	location = __ua_get_location_info_from_list(
		info->service_handle, info->device_handle);

	if (location) {
		ret = __ua_update_location_info(location, info);
		goto done;
	}

	location = __ua_allocate_ua_location_info();
	if (!location)
		return UA_ERROR_OUT_OF_MEMORY;

	ret = __ua_update_location_info(location, info);
	ua_location_list = g_slist_append(ua_location_list, location);

done:
	FUNC_EXIT;
	return ret;
}

static void __send_location_cb(gpointer data, gpointer user_data)
{
	FUNC_ENTRY;
	ua_location_h location_handle = NULL;
	ua_location_info_s *location_info = data;
	ua_service_h service_handle = user_data;

	ret_if(!location_info);
	ret_if(!service_handle);

	ua_monitor_s *monitor = (ua_monitor_s *)(location_info->monitor_handle);

	if (!monitor || !(monitor->location_cb))
		return;

	if (!(location_info->service_handle == service_handle))
		return;

	/* Get location handle */
	location_handle = (ua_location_h)location_info;

	/* Send location callback */
	monitor->location_cb(UA_ERROR_NONE, monitor,
		location_info->service_handle, location_info->bitmask,
		location_info->sensor_handle, location_info->device_handle,
		location_info->user_handle, location_handle,
		monitor->user_data);

	return;
}

gboolean _ua_monitor_send_location_cb(gpointer data)
{
	FUNC_ENTRY;

	g_slist_foreach(ua_location_list, __send_location_cb, data);

	FUNC_EXIT;
	return TRUE;
}

static void __ua_free_ua_location_info_s(gpointer data)
{
	FUNC_ENTRY;
	ua_location_info_s *location_info = (ua_location_info_s*)data;

	if (!location_info)
		return;
	g_free(location_info);
	FUNC_EXIT;
}

void _ua_clear_location_list(void)
{
	g_slist_free_full(ua_location_list, __ua_free_ua_location_info_s);
	ua_location_list = NULL;
}
