/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <user-awareness.h>

#include "uat-menu.h"
#include "uat-common.h"

#define CASE_TO_STR(x) case x: return #x;
#define UA_BLE_PAYLOAD_DEVICE_UID_MAX_LEN 17

const char* uat_get_error_str(int err)
{
	switch (err) {
	/* CHECK: List all enum values here */
	CASE_TO_STR(UA_ERROR_NONE)
	CASE_TO_STR(UA_ERROR_NOT_PERMITTED)
	CASE_TO_STR(UA_ERROR_INVALID_PARAMETER)
	CASE_TO_STR(UA_ERROR_OUT_OF_MEMORY)
	CASE_TO_STR(UA_ERROR_PERMISSION_DENIED)
	CASE_TO_STR(UA_ERROR_RESOURCE_BUSY)
	CASE_TO_STR(UA_ERROR_TIMED_OUT)
	CASE_TO_STR(UA_ERROR_CANCELED)
	CASE_TO_STR(UA_ERROR_NOW_IN_PROGRESS)
	CASE_TO_STR(UA_ERROR_NOT_SUPPORTED)
	CASE_TO_STR(UA_ERROR_NO_DATA)
	CASE_TO_STR(UA_ERROR_NOT_INITIALIZED)
	CASE_TO_STR(UA_ERROR_NOT_IN_PROGRESS)
	CASE_TO_STR(UA_ERROR_ALREADY_DONE)
	CASE_TO_STR(UA_ERROR_OPERATION_FAILED)
	CASE_TO_STR(UA_ERROR_NOT_READY)
	CASE_TO_STR(UA_ERROR_NOT_ENABLED)
	CASE_TO_STR(UA_ERROR_NOT_FOUND)
	CASE_TO_STR(UA_ERROR_ALREADY_REGISTERED)
	CASE_TO_STR(UA_ERROR_DB_FAILED)
	CASE_TO_STR(UA_ERROR_NOT_REGISTERED)
	default :
		return "Unknown Error";
	}
}

const char* uat_get_sensor_str(int sensor)
{
	switch (sensor) {
	/* CHECK: List all enum values here */
	case UAT_SENSOR_TYPE_BT:
		return "BT";
	case UAT_SENSOR_TYPE_BLE:
		return "BLE";
	case UAT_SENSOR_TYPE_WIFI:
		return "WIFI";
	case UAT_SENSOR_TYPE_MOTION:
		return "MOTION";
	case UAT_SENSOR_TYPE_LIGHT:
		return "LIGHT";
	case UAT_SENSOR_TYPE_AUDIO:
		return "AUDIO";
	default:
		return "Unknown Sesnor";
	}

}

const char* uat_get_mac_type_str(int sensor)
{
	switch (sensor) {
	/* CHECK: List all enum values here */
	case UA_MAC_TYPE_NONE:
		return "NONE";
	case UA_MAC_TYPE_BT:
		return "BT";
	case UA_MAC_TYPE_BLE:
		return "BLE";
	case UA_MAC_TYPE_WIFI:
		return "WIFI";
	case UA_MAC_TYPE_P2P:
		return "P2P";
	case UA_MAC_TYPE_WIFI_LOCATION:
		return "WIFI-LOCATION";
	case UA_MAC_TYPE_UWB:
		return "UWB";
	default:
		return "Unknown MAC type";
	}
}

const char* uat_get_str_from_uat_mac_type(int mac_type)
{
	switch (mac_type) {
	/* CHECK: List all enum values here */
	case UAT_MAC_TYPE_BT:
		return "BT";
	case UAT_MAC_TYPE_BLE:
		return "BLE";
	case UAT_MAC_TYPE_WIFI:
		return "WIFI";
	case UAT_MAC_TYPE_P2P:
		return "P2P";
	default:
		return "Unknown MAC type";
	}
}

const char* uat_get_sensor_bitmask_str(int sensor)
{
	switch (sensor) {
	/* CHECK: List all enum values here */
	case UA_SENSOR_BT:
		return "BT";
	case UA_SENSOR_BLE:
		return "BLE";
	case UA_SENSOR_WIFI:
		return "WIFI";
	case UA_SENSOR_MOTION:
		return "MOTION";
	case UA_SENSOR_LIGHT:
		return "LIGHT";
	case UA_SENSOR_AUDIO:
		return "AUDIO";
	case UA_SENSOR_WIFI_LOCATION:
		return "WIFI-LOCATION";
	case UA_SENSOR_UWB:
		return "UWB";
	default:
		return "Unknown";
	}
}

char* uat_get_time()
{
	time_t t;
	struct tm time_deysel;

	char *pbuf = calloc(128, sizeof(char));
	if (!pbuf)
		return NULL;

	t = time(NULL);
	localtime_r(&t, &time_deysel);

	snprintf(pbuf, 128, "%04d-%02d-%02d %02d:%02d:%02d",
			time_deysel.tm_year + 1900, time_deysel.tm_mon + 1, time_deysel.tm_mday,
			time_deysel.tm_hour, time_deysel.tm_min, time_deysel.tm_sec);

	return pbuf;
}


void uat_print_device_uid(const char *device_uid)
{
	fprintf(stdout, ANSI_COLOR_WHITE "payload device_uid :" ANSI_COLOR_NORMAL);
	if (!device_uid)
		goto done;

	for (int i = 0; i < UA_BLE_PAYLOAD_DEVICE_UID_MAX_LEN; i++)
		fprintf(stdout, ANSI_COLOR_WHITE " %d" ANSI_COLOR_NORMAL, device_uid[i]);

done:
	fprintf(stdout, "\n");
	fflush(stdout);
}

static void  _free_device_handle(gpointer data)
{
	int ret;

	ua_device_h handle = data;
	if (handle) {
		ret = ua_device_destroy(handle);
		handle = NULL;

		msg(" - ua_device_destroy() ret: [0x%X] [%s]",
				ret, uat_get_error_str(ret));
        }
}

GSList *g_device_list = NULL; /**< Device List */
int g_device_count = 0; /**< Seletected service name */

void uat_clear_device_list(void)
{
	if (NULL == g_device_list) {
		msgr("g_device_list is NULL");
		return;
	}
	g_slist_free_full(g_device_list, _free_device_handle);
	g_device_list = NULL;
	g_device_count = 0;
}

ua_mac_type_e uat_convert_idx_to_device_type(int idx)
{
	switch (idx) {
	case UAT_MAC_TYPE_BT:
		return UA_MAC_TYPE_BT;
	case UAT_MAC_TYPE_BLE:
		return UA_MAC_TYPE_BLE;
	case UAT_MAC_TYPE_WIFI:
		return UA_MAC_TYPE_WIFI;
	case UAT_MAC_TYPE_P2P:
		return UA_MAC_TYPE_P2P;
	default:
		return UA_MAC_TYPE_INVALID;
	}
}

uat_mac_type_e uat_convert_device_type_to_idx(int mac_type)
{
	switch (mac_type) {
	case UA_MAC_TYPE_BT:
		return UAT_MAC_TYPE_BT;
	case UA_MAC_TYPE_BLE:
		return UAT_MAC_TYPE_BLE;
	case UA_MAC_TYPE_WIFI:
		return UAT_MAC_TYPE_WIFI;
	case UA_MAC_TYPE_P2P:
		return UAT_MAC_TYPE_P2P;
	default:
		return UAT_MAC_TYPE_MAX;
	}
}
