/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <gio/gio.h>

#include <user-awareness.h>
#include <user-awareness-internal.h>

#include "uat-menu.h"
#include "uat-common.h"

extern ua_monitor_h g_ua_mon_h; /**< Monitor handle */
ua_device_h g_device_h = NULL; /**< Device handle */
extern ua_user_h g_user_h; /**< User handle */
extern ua_service_h g_service_h; /**< Service handle */

char g_device_type[MENU_DATA_SIZE + 1] = {"3"}; /**< Sensor type of the selected device */
char g_device_type_str[MENU_DATA_SIZE + 1] = { "WIFI" }; /**< Readable sensor type string */

static char g_os_type[MENU_DATA_SIZE + 1] = {"2"}; /**< OS type of the selected device */
char g_os_type_str[MENU_DATA_SIZE + 1] = { "Android" }; /**< Readable OS type string */

char g_device_id_str[MENU_DATA_SIZE + 1] = {0,}; /**< Mobile ID of the selected device */
char g_mac_addr_str[MENU_DATA_SIZE + 1] = {0,}; /**< MAC of the selected device */
static char g_bss_id_str[MENU_DATA_SIZE + 1] = {0,}; /**< BSSID of the selected device */
static char g_ipv4_address_str[MENU_DATA_SIZE + 1] = {0,}; /**< IPv4 of the selected device */
static char g_is_paring_required_str[MENU_DATA_SIZE + 1] = {0,}; /**< Is paring required for the selected device */

char g_selected_device_idx[MENU_DATA_SIZE + 1] = {0,}; /**< Selected device id */
extern char g_service_str[MENU_DATA_SIZE + 1];  /**< Service name string */
extern char g_user_account_str[MENU_DATA_SIZE + 1]; /**< Find device by user ID */

extern GSList *g_device_list; /**< Device List */
extern int g_device_count; /**< Seletected service name */

bool _foreach_added_device_cb(
	ua_device_h device_handle, void *user_data)
{
	int  ret_temp;

	ua_mac_type_e mac_type = UA_MAC_TYPE_INVALID;
	ua_os_type_e ostype = UA_OS_TYPE_NOT_DEFINED;
	char *device_id = NULL;
	char *mac = NULL;
	char *wifi_bssid = NULL;
	char *ipv4 = NULL;
	bool required = false;
	unsigned long long last_seen = 0;

	if (!device_handle) {
		msgr("device_handle is NULL");
		return true;
	}

	msgb("\n[%d]", ++g_device_count);

	ret_temp = ua_device_get_mac_type(device_handle, &mac_type);
	if (UA_ERROR_NONE == ret_temp)
		msgb("Device MAC Type : %s", uat_get_mac_type_str(mac_type));

	ret_temp = ua_device_get_os_info(device_handle, &ostype);
	if (UA_ERROR_NONE == ret_temp) {
		msgb("Device OS Type : %s",
			ostype == 1 ? "Tizen" :
			ostype == 2 ? "Android" :
			ostype == 3 ? "iOS" : "N/A");
	}

	ret_temp = ua_device_get_device_id(device_handle, &device_id);
	if (UA_ERROR_NONE == ret_temp) {
		msgb("Device ID : %s", device_id);
		g_free(device_id);
	}

	ret_temp = ua_device_get_mac_address(device_handle, &mac);
	if (UA_ERROR_NONE == ret_temp) {
		msgb("Device MAC : %s", mac);
		g_free(mac);
	}

	ret_temp = ua_device_get_wifi_bssid(device_handle, &wifi_bssid);
	if (UA_ERROR_NONE == ret_temp) {
		msgb("Device BSSID : %s", wifi_bssid);
		g_free(wifi_bssid);
	}

	ret_temp = ua_device_get_wifi_ipv4_address(device_handle, &ipv4);
	if (UA_ERROR_NONE == ret_temp) {
		msgb("Device IPv4 Addr : %s", ipv4);
		g_free(ipv4);
	}

	ret_temp = ua_device_get_pairing_required(device_handle, &required);
	if (UA_ERROR_NONE == ret_temp)
		msgb("Pairing Required : %s", required ? "YES" : "NO");

	ret_temp = ua_device_get_last_presence(device_handle, &last_seen);
	if (UA_ERROR_NONE == ret_temp)
		msgb("Last present at : %llu", last_seen);

	g_device_list = g_slist_append(g_device_list, device_handle);

	return true;
}

void uat_update_device_info(void)
{
	int ret = UA_ERROR_NONE;
	ua_mac_type_e mac_type = UA_MAC_TYPE_INVALID;
	ua_os_type_e ostype = UA_OS_TYPE_NOT_DEFINED;
	char *device_id = NULL;
	char *mac = NULL;
	char *wifi_bssid = NULL;
	char *ipv4 = NULL;
	bool required = false;

	check_if(NULL == g_device_h);

	ret = ua_device_get_mac_type(g_device_h, &mac_type);
	if (UA_ERROR_NONE == ret) {
		memset(g_device_type_str, 0, MENU_DATA_SIZE + 1);
		snprintf(g_device_type_str, MENU_DATA_SIZE, "%s",
			uat_get_mac_type_str(mac_type));
		snprintf(g_device_type, MENU_DATA_SIZE, "%d",
			uat_convert_device_type_to_idx(mac_type));
	}

	ret = ua_device_get_os_info(g_device_h, &ostype);
	snprintf(g_os_type_str, MENU_DATA_SIZE, "%s",
		ostype == 1 ? "Tizen" :
		ostype == 2 ? "Android" :
		ostype == 3 ? "iOS" : "N/A");

	ret = ua_device_get_device_id(g_device_h, &device_id);
	if (UA_ERROR_NONE == ret && device_id) {
		memset(g_device_id_str, 0, MENU_DATA_SIZE + 1);
		memcpy(g_device_id_str, device_id, MENU_DATA_SIZE);
		g_free(device_id);
	}

	ret = ua_device_get_mac_address(g_device_h, &mac);
	if (UA_ERROR_NONE == ret && mac) {
		memset(g_mac_addr_str, 0, MENU_DATA_SIZE + 1);
		memcpy(g_mac_addr_str, mac, MENU_DATA_SIZE);
		g_free(mac);
	}

	ret = ua_device_get_wifi_bssid(g_device_h, &wifi_bssid);
	if (UA_ERROR_NONE == ret && wifi_bssid) {
		memset(g_bss_id_str, 0, MENU_DATA_SIZE + 1);
		memcpy(g_bss_id_str, wifi_bssid, MENU_DATA_SIZE);
		g_free(wifi_bssid);
	}

	ret = ua_device_get_wifi_ipv4_address(g_device_h, &ipv4);
	if (UA_ERROR_NONE == ret && ipv4) {
		memset(g_ipv4_address_str, 0, MENU_DATA_SIZE + 1);
		memcpy(g_ipv4_address_str, ipv4, MENU_DATA_SIZE);
		g_free(ipv4);
	}

	ret = ua_device_get_pairing_required(g_device_h, &required);
	if (UA_ERROR_NONE == ret) {
		memset(g_is_paring_required_str, 0, MENU_DATA_SIZE + 1);
		memcpy(g_is_paring_required_str,
			required ? "YES" : "NO ", sizeof("YES"));
	}

}

static int run_ua_device_create(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	uat_mac_type_e mac_type = UAT_MAC_TYPE_MAX;

	msg("ua_device_create");

	if (g_device_h) {
		ret = ua_device_destroy(g_device_h);
		g_device_h = NULL;

		msg(" - ua_device_destroy() ret: [0x%X] [%s]",
				ret, uat_get_error_str(ret));
	}

	if (strlen(g_device_type)) {
		mac_type = (unsigned char)strtol(g_device_type, NULL, 10);
		snprintf(g_device_type_str, MENU_DATA_SIZE, "%s",
			uat_get_str_from_uat_mac_type(mac_type));
	}

	ret = ua_device_create(uat_convert_idx_to_device_type(mac_type),
				g_mac_addr_str, g_device_id_str, &g_device_h);

	msg(" - ua_device_create() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_device_destroy(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_device_destroy");

	ret = ua_device_destroy(g_device_h);
	g_device_h = NULL;

	snprintf(g_device_type_str, MENU_DATA_SIZE, "N/A");

	msg(" - ua_device_destroy() ret: [0x%X] [%s]",
		ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_device_set_os_info(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	ua_os_type_e ostype = UA_OS_TYPE_NOT_DEFINED;

	msg("ua_device_set_os_info");

	if (strlen(g_os_type))
		ostype = (unsigned char)strtol(g_os_type, NULL, 10);

	if (ostype > UA_OS_TYPE_IOS)
		ostype = UA_OS_TYPE_IOS;
	else if (ostype < UA_OS_TYPE_NOT_DEFINED)
		ostype = UA_OS_TYPE_TIZEN;

	snprintf(g_os_type_str, MENU_DATA_SIZE, "%s",
		ostype == 1 ? "Tizen" :
		ostype == 2 ? "Android" :
		ostype == 3 ? "iOS" : "N/A");

	ret = ua_device_set_os_info(g_device_h, ostype);

	msg(" - ua_device_set_os_info() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_device_get_wifi_bssid(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	char *bssid = NULL;

	msg("ua_device_get_wifi_bssid");

	ret = ua_device_get_wifi_bssid(g_device_h, &bssid);
	if (UA_ERROR_NONE == ret) {
		memset(g_bss_id_str, 0, MENU_DATA_SIZE + 1);
		memcpy(g_bss_id_str, bssid, MENU_DATA_SIZE);
		g_free(bssid);
	}

	msg(" - ua_device_get_wifi_bssid() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_device_set_wifi_ipv4_address(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_device_set_wifi_ipv4_address");

	ret = ua_device_set_wifi_ipv4_address(g_device_h, g_ipv4_address_str);

	msg(" - ua_device_set_wifi_ipv4_address() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_device_get_by_mac_address(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	ua_device_h _device = NULL;

	msg("ua_device_get_by_mac_address");

	ret = ua_device_get_by_mac_address(g_mac_addr_str, &_device);
	if (UA_ERROR_NONE == ret) {
		g_device_h = _device;
		uat_update_device_info();

	}

	msg(" - ua_device_get_by_mac_address() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_device_get_by_device_id(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	ua_device_h _device = NULL;
	ua_mac_type_e mac_type = UA_MAC_TYPE_NONE;

	msg("ua_device_get_by_device_id");

	if (strlen(g_device_type)) {
		int temp = (unsigned char)strtol(g_device_type, NULL, 10);
		mac_type = uat_convert_idx_to_device_type(temp);
		snprintf(g_device_type_str, MENU_DATA_SIZE + 1, "%s",
			uat_get_mac_type_str(mac_type));
	}
	msg("device_id [%s] mac_type [%s]", g_device_id_str, g_device_type_str);

	ret = ua_device_get_by_device_id(g_device_id_str, mac_type, &_device);
	if (UA_ERROR_NONE == ret) {
		g_device_h = _device;
		uat_update_device_info();
	}

	msg(" - ua_device_get_by_device_id() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_foreach_devices(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_foreach_devices");

	uat_clear_device_list();

	ret = ua_foreach_devices(_foreach_added_device_cb, NULL);

	msg(" - ua_foreach_devices() ret: [0x%X] [%s]",
		ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_device_update(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_device_update");

	ret = ua_device_update(g_device_h);

	msg(" - ua_device_update() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static struct menu_data menu_ua_device_create[] = {
	{ "1", "device_id",
		NULL, NULL, g_device_id_str },
	{ "2", "type (1:BT 2:BLE 3:Wi-Fi 4:P2P)",
		NULL, NULL, g_device_type },
	{ "3", "MAC",
		NULL, NULL, g_mac_addr_str },
	{ "4", "run", NULL,
		run_ua_device_create, NULL },
	{ NULL, NULL, },
};

static struct menu_data menu_ua_os_type[] = {
	{ "1", "type (1:Tizen 2:Andorid 3:iOS)",
		NULL, NULL, g_os_type },
	{ "2", "run", NULL,
		run_ua_device_set_os_info, NULL },
	{ NULL, NULL, },
};

static struct menu_data menu_ua_ipv4_address[] = {
	{ "1", "IPv4 address",
		NULL, NULL, g_ipv4_address_str },
	{ "2", "run", NULL,
		run_ua_device_set_wifi_ipv4_address, NULL },
	{ NULL, NULL, },
};

static struct menu_data menu_ua_devlist_by_mac[] = {
	{ "1", "MAC",
		NULL, NULL, g_mac_addr_str },
	{ "2", "run", NULL,
		run_ua_device_get_by_mac_address, NULL },
	{ NULL, NULL, },
};

static struct menu_data menu_ua_devlist_by_device_id[] = {
	{ "1", "type (1:BT 2:BLE 3:Wi-Fi 4:P2P)",
		NULL, NULL, g_device_type },
	{ "2", "device_id",
		NULL, NULL, g_device_id_str },
	{ "3", "run", NULL,
		run_ua_device_get_by_device_id, NULL },
	{ NULL, NULL, },
};

struct menu_data menu_ua_devices[] = {
	{ "1", "ua_device_create",
		menu_ua_device_create, NULL, NULL},
	{ "2", "ua_device_destroy",
		NULL, run_ua_device_destroy, NULL },
	{ "3", "ua_device_set_os_info",
		menu_ua_os_type, NULL, g_os_type_str },
	{ "5", "ua_device_set_wifi_ipv4_address",
		menu_ua_ipv4_address, NULL, g_ipv4_address_str },
	{ "11", "ua_device_get_wifi_bssid",
		NULL, run_ua_device_get_wifi_bssid, NULL },
	{ "12", "ua_device_get_by_mac_address",
		menu_ua_devlist_by_mac, NULL, NULL },
	{ "13", "ua_device_get_by_device_id",
		menu_ua_devlist_by_device_id, NULL, NULL },
	{ "14", "ua_device_update",
		NULL, run_ua_device_update, NULL },
	{ "15", "ua_foreach_devices",
		NULL, run_ua_foreach_devices, NULL },
	{ NULL, NULL, },
};

int uat_select_device(MManager *mm, struct menu_data *menu)
{
	GSList *iter = g_device_list;

	int id = 0;
	int selected_id = 0;

	if (strlen(g_selected_device_idx))
		selected_id = (unsigned char)strtol(g_selected_device_idx, NULL, 10);

	if (selected_id <= 0) {
		msg("Please select device first");
		return RET_SUCCESS;
	}

	while (NULL != iter) {
		GSList *next = iter->next;
		ua_device_h *handle = iter->data;
		if (handle && ++id == selected_id) {
			/* Make selected device as a current one */
			ua_device_clone(&g_device_h, handle);
			/* Update device info. */
			uat_update_device_info();
			msg("[%d] device selected", selected_id);
			return RET_SUCCESS;
		}
		iter = next;
	}

	return RET_SUCCESS;
}

struct menu_data menu_sel_device[] = {
	{ "1", "Device list", NULL, run_ua_foreach_devices, g_selected_device_idx},
	{ "2", "Apply", NULL, uat_select_device, NULL },
	{ NULL, NULL, },
};
