/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <gio/gio.h>

#include <user-awareness.h>
#include <user-awareness-internal.h>

#include "uat-menu.h"
#include "uat-common.h"

extern ua_monitor_h g_ua_mon_h; /**< Monitor handle */
extern ua_service_h g_service_h; /**< Service handle */
extern char g_service_str[MENU_DATA_SIZE + 1]; /**< Service name */
extern char g_sensor_list_str[MENU_DATA_SIZE + 1]; /**< Sendosr list string */
extern unsigned int g_sensor_list; /**< Added sensor list */

const char * uat_convert_device_mac_type_to_txt(int mac_type)
{
	switch (mac_type) {
	case UA_MAC_TYPE_BT:
		return "BT";
	case UA_MAC_TYPE_BLE:
		return "BLE";
	case UA_MAC_TYPE_WIFI:
		return "WIFI";
	case UA_MAC_TYPE_P2P:
		return "P2P";
	case UA_MAC_TYPE_WIFI_LOCATION:
		return "WIFI-LOCATION";
	case UA_MAC_TYPE_UWB:
		return "UWB";
	default:
		return "UNKNOWN";
	}
}

static void __sensor_status_changed_cb(ua_monitor_h handle,
	ua_sensor_h sensor_handle, ua_sensor_type_e sensor_type,
	ua_sensor_report_e status, void *user_data)
{
	int ret;
	unsigned long long timestamp;
	char *pbuf = uat_get_time();

	msgc("\n[%s]", pbuf);
	free(pbuf);

	msgc("[%s] Sensor Status Changed -> %s",
		uat_get_sensor_bitmask_str(sensor_type),
		status == UA_SENSOR_REPORT_PRESENCE ? "PRESENCE" : "ABSENCE");

	ret = ua_sensor_get_timestamp(sensor_handle, &timestamp);
	if (UA_ERROR_NONE != ret) {
		msg(" - ua_sensor_get_timestamp() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));
	}

	msgc("[%s] timestamp [%llu]",
		uat_get_sensor_bitmask_str(sensor_type), timestamp);
}

int _uat_monitor_create()
{
	const char *buf = NULL;
	int ret = UA_ERROR_NONE;
	bool is_available = false;

	msg("ua_monitor_create");
	if (g_ua_mon_h) {
		ret = ua_monitor_destroy(g_ua_mon_h);
		msg(" - ua_monitor_distroy() ret: [0x%X] [%s]", ret, uat_get_error_str(ret));
		g_ua_mon_h = NULL;
	}

	ret = ua_monitor_create(&g_ua_mon_h);
	msg(" - ua_monitor_create() ret: [0x%X] [%s]", ret, uat_get_error_str(ret));

	if (UA_ERROR_NONE != ret)
		return ret;

	memset(g_sensor_list_str, 0, MENU_DATA_SIZE + 1);
	for (unsigned int i = UA_SENSOR_BT ; i < UA_SENSOR_MAX; i <<= 1) {
		ret = ua_monitor_is_sensor_available(i, &is_available);
		if (UA_ERROR_NONE == ret && is_available) {

			/* If sensor plugins are loaded, enable them */
			ret = ua_monitor_add_sensor(g_ua_mon_h, i);
			msg(" - ua_monitor_add_sensor() ret: [0x%X] [%s]", ret, uat_get_error_str(ret));

			/* Save loaded sensor */
			g_sensor_list |= i;

			buf = uat_get_sensor_bitmask_str(i);
			strncat(g_sensor_list_str, buf,
					sizeof(g_sensor_list_str) - strlen(g_sensor_list_str) - 1);
			strncat(g_sensor_list_str, " ",
					sizeof(g_sensor_list_str) - strlen(g_sensor_list_str) - 1);

		}
		is_available = false;
	}

	/* Add Wifi location Sensor */
	unsigned int i;
	i = UA_SENSOR_WIFI_LOCATION;
	ret = ua_monitor_is_sensor_available(i, &is_available);
	if (UA_ERROR_NONE == ret && is_available) {
		ua_monitor_add_sensor(g_ua_mon_h, i);
		g_sensor_list |= i;
		buf = uat_get_sensor_bitmask_str(i);
		strncat(g_sensor_list_str, buf,
				sizeof(g_sensor_list_str) - strlen(g_sensor_list_str) - 1);
		strncat(g_sensor_list_str, " ",
				sizeof(g_sensor_list_str) - strlen(g_sensor_list_str) - 1);
	}

	/* Add UWB Sensor */
	i = UA_SENSOR_UWB;
	ret = ua_monitor_is_sensor_available(i, &is_available);
	if (UA_ERROR_NONE == ret && is_available) {
		ua_monitor_add_sensor(g_ua_mon_h, i);
		g_sensor_list |= i;
		buf = uat_get_sensor_bitmask_str(i);
		strncat(g_sensor_list_str, buf,
				sizeof(g_sensor_list_str) - strlen(g_sensor_list_str) - 1);
		strncat(g_sensor_list_str, " ",
				sizeof(g_sensor_list_str) - strlen(g_sensor_list_str) - 1);
	}

	return ret;
}

static int run_ua_monitor_create(MManager *mm, struct menu_data *menu)
{
	_uat_monitor_create();

	return RET_SUCCESS;
}

static int run_ua_monitor_destroy(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_monitor_destroy");
	if (g_ua_mon_h)
		ret = ua_monitor_destroy(g_ua_mon_h);

	msg(" - ua_monitor_destroy() ret: [0x%X] [%s]", ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

int _uat_monitor_set_sensor_status_cb()
{
	int ret = UA_ERROR_NONE;

	msg("ua_monitor_set_sensor_status_changed_cb");

	ret = ua_monitor_set_sensor_status_changed_cb(g_ua_mon_h,
			__sensor_status_changed_cb, g_ua_mon_h);

	msg(" - ua_monitor_set_sensor_status_changed_cb() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return ret;
}

static int run_ua_monitor_set_sensor_status_changed_cb(
		MManager *mm, struct menu_data *menu)
{
	_uat_monitor_set_sensor_status_cb();

	return RET_SUCCESS;
}

static int run_ua_monitor_unset_sensor_status_changed_cb(
		MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_monitor_unset_sensor_status_changed_cb");

	ret = ua_monitor_unset_sensor_status_changed_cb(g_ua_mon_h);

	msg(" - ua_monitor_unset_sensor_status_changed_cb() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_reset_database(
		MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_reset_database");

	ret = ua_reset_database();

	msg(" - ua_reset_database() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

struct menu_data menu_ua_init[] = {
	{ "1", "ua_monitor_create",
		NULL, run_ua_monitor_create, NULL },
	{ "2", "ua_monitor_destroy",
		NULL, run_ua_monitor_destroy, NULL },
	{ "3", "ua_monitor_set_sensor_status_changed_cb",
		NULL, run_ua_monitor_set_sensor_status_changed_cb, NULL },
	{ "4", "ua_monitor_unset_sensor_status_changed_cb",
		NULL, run_ua_monitor_unset_sensor_status_changed_cb, NULL },
	{ "5", "ua_reset_database",
		NULL, run_ua_reset_database, NULL},
	{ NULL, NULL, },
};
