/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <gio/gio.h>

#include <user-awareness.h>
#include <user-awareness-internal.h>

#include "uat-menu.h"
#include "uat-common.h"

extern ua_monitor_h g_ua_mon_h; /**< Monitor handle */
extern ua_device_h g_device_h; /**< Device handle */
ua_user_h g_user_h = NULL; /**< User handle */
extern ua_service_h g_service_h; /**< Service handle */

char g_user_account_str[MENU_DATA_SIZE + 1] = { "default@default.com" };
char g_user_name_str[MENU_DATA_SIZE + 1] = { "default" };

extern char g_device_type[MENU_DATA_SIZE + 1]; /**< Sensor type of the selected device */
extern char g_device_type_str[MENU_DATA_SIZE + 1]; /**< Readable sensor type string */
extern char g_device_id_str[MENU_DATA_SIZE + 1]; /**< Mobile ID of the selected device */
extern char g_mac_addr_str[MENU_DATA_SIZE + 1]; /**< MAC of the selected device */

extern char g_service_str[MENU_DATA_SIZE + 1]; /**< Service name */

GSList *user_list = NULL; /**< User List */
int idx_user = 0; /**< Found user sequence number */

extern char g_selected_device_idx[MENU_DATA_SIZE + 1]; /**< Selected device id */
extern GSList *g_device_list; /**< Device List */
extern int g_device_count; /**< Seletected service name */

char g_selected_user_idx[MENU_DATA_SIZE + 1] = {0,}; /** Selected user id */

static void update_user_info(void)
{
	int ret = UA_ERROR_NONE;
	char *account = NULL;

	if (0 == idx_user) {
		if (g_user_h) {
			ua_user_destroy(g_user_h);
			g_user_h = NULL;
		}
		ua_get_default_user(&g_user_h);
	}

	ret  = ua_user_get_account(g_user_h, &account);
	if (UA_ERROR_NONE == ret) {
		memset(g_user_account_str, 0, MENU_DATA_SIZE + 1);
		memcpy(g_user_account_str, account, MENU_DATA_SIZE);
		g_free(account);
	}
}

static void  ___free_users(gpointer data)
{
	ua_user_h handle = data;
	ua_user_destroy(handle);
	handle = NULL;
}

void uat_clear_user_list(void)
{
	g_slist_free_full(user_list, ___free_users);
	user_list = NULL;
	idx_user = 0;
}

bool _foreach_registered_user_cb(
	ua_user_h user_handle, void *user_data)
{
	int ret;
	char *account = NULL;

	if (!user_handle) {
		msgr("user_handle is NULL");
		return true;
	}

	user_list = g_slist_append(user_list, user_handle);
	idx_user++;

	ret = ua_user_get_account(user_handle, &account);
	if (UA_ERROR_NONE == ret) {
		msglr("[%d]", idx_user);
		msglr("User account info %s", account);
		g_free(account);
	}

	return true;
}

void _print_device_info(ua_device_h handle)
{
	int ret;
	ua_mac_type_e mac_type = UA_MAC_TYPE_INVALID;
	ua_os_type_e ostype = UA_OS_TYPE_NOT_DEFINED;
	char *device_id = NULL;
	char *mac = NULL;
	char *ipv4 = NULL;

	if (NULL == handle) {
		msgr("Device handle is NULL");
		return;
	}

	ret = ua_device_get_mac_type(handle, &mac_type);
	if (UA_ERROR_NONE == ret)
		msg("Device MAC Type : %s", uat_get_mac_type_str(mac_type));

	ret = ua_device_get_os_info(handle, &ostype);
	if (UA_ERROR_NONE == ret)
		msg("Device OS Type : %s",
			ostype == 1 ? "Tizen" :
			ostype == 2 ? "Android" :
			ostype == 3 ? "iOS" : "N/A");

	ret = ua_device_get_device_id(handle, &device_id);
	if (UA_ERROR_NONE == ret) {
		msg("Device ID : %s", device_id);
		g_free(device_id);
	}

	ret = ua_device_get_mac_address(handle, &mac);
	if (UA_ERROR_NONE == ret) {
		msg("Device MAC : %s", mac);
		g_free(mac);
	}

	ret = ua_device_get_wifi_ipv4_address(handle, &ipv4);
	if (UA_ERROR_NONE == ret) {
		msg("Device IPv4 Addr : %s", ipv4);
		g_free(ipv4);
	}
}

static void __device_added_cb(int result,
		ua_device_h handle, void *user_data)
{
	if (UA_ERROR_NONE != result)
		msgr("__device_added_cb() result: [0x%X] [%s]",
			result, uat_get_error_str(result));
	else
		msgb("__device_added_cb() result: [0x%X] [%s]",
			result, uat_get_error_str(result));

	_print_device_info(handle);
}

static bool _user_foreach_added_device_cb(ua_device_h device_handle,
	void *user_data)
{
	if (!device_handle) {
		msgr("device_handle is NULL");
		return true;
	}

	msgc("\n[%d]", ++g_device_count);

	_print_device_info(device_handle);
	g_device_list = g_slist_append(g_device_list, device_handle);

	return true;
}

int uat_select_user(MManager *mm, struct menu_data *menu)
{
	GSList *iter = user_list;

	int id = 0;
	int selected_id = 0;

	if (strlen(g_selected_user_idx))
		selected_id = (unsigned char)strtol(g_selected_user_idx, NULL, 10);

	if (selected_id <= 0) {
		msg("Please select user first");
		return RET_SUCCESS;
	}

	while (NULL != iter) {
		GSList *next = iter->next;
		ua_user_h *handle = iter->data;
		if (handle && ++id == selected_id) {
			g_user_h = handle; /* Make selected user as a current one */
			update_user_info(); /* Update user info. */
			msg("Select user done");
			return RET_SUCCESS;
		}
		iter = next;
	}

	return RET_SUCCESS;
}

int _uat_user_create(char* user_account_str)
{
	int ret = UA_ERROR_NONE;

	msg("ua_user_create");

	if (g_user_h) {
		ua_user_destroy((g_user_h));
		g_user_h = NULL;
	}

	ret = ua_user_create(user_account_str, &g_user_h);

	msg(" - ua_user_create() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return ret;
}

static int run_ua_user_create(MManager *mm, struct menu_data *menu)
{
	_uat_user_create(g_user_account_str);

	return RET_SUCCESS;
}

static int run_ua_user_destroy(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_user_destroy");

	if (g_user_h) {
		ret = ua_user_destroy((g_user_h));
		g_user_h = NULL;
	}

	msg(" - ua_user_destroy() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_user_add(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_user_add");

	check_if(NULL == g_user_h);

	ret = ua_user_add(g_user_h);

	msg(" - ua_user_add() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_user_remove(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_user_remove");

	check_if(NULL == g_user_h);

	ret = ua_user_remove(g_user_h);

	msg(" - ua_user_remove() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_get_default_user(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_get_default_user");

	if (g_user_h) {
		ua_user_destroy(g_user_h);
		g_user_h = NULL;
	}

	ret = ua_get_default_user(&g_user_h);
	if (UA_ERROR_NONE == ret)
		update_user_info();

	msg(" - ua_get_default_user() ret: [0x%X] [%s]",
		ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_get_user_by_account(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	ua_user_h user_handle;

	msg("ua_service_get_user_by_account");

	ret = ua_service_get_user_by_account(g_service_h,
				g_user_account_str, &user_handle);

	msg(" - ua_service_get_user_by_account() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	if (UA_ERROR_NONE == ret) {
		msgb("User Handle [%p]", user_handle);
		ua_user_destroy(g_user_h);
		g_user_h = user_handle;
		update_user_info();
	}
	return RET_SUCCESS;
}

static int run_ua_user_foreach(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	msg("ua_foreach_users");

	uat_clear_user_list();

	ret = ua_foreach_users(_foreach_registered_user_cb, NULL);

	msg(" - ua_foreach_users() ret: [0x%X] [%s]",
				ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_user_add_device(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_user_add_device");

	check_if(NULL == g_device_h);

	ret = ua_user_add_device(g_user_h, g_device_h, __device_added_cb, NULL);

	msg(" - ua_user_add_device() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_user_remove_device(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_user_remove_device");

	check_if(NULL == g_user_h);
	check_if(NULL == g_device_h);

	ret = ua_user_remove_device(g_user_h, g_device_h);

	msg(" - ua_user_remove_device() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_user_foreach_devices(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	ua_user_h user_handle;

	msg("ua_user_foreach_devices");

	ret = ua_service_get_user_by_account(g_service_h,
				g_user_account_str, &user_handle);
	if (UA_ERROR_NONE == ret) {
		msgb("User Handle [%p]", user_handle);
		ua_user_destroy(g_user_h);
		g_user_h = user_handle;
		update_user_info();
	} else {
		msg(" - ua_service_get_user_by_account() ret: [0x%X] [%s]",
				ret, uat_get_error_str(ret));
	}

	uat_clear_device_list();

	ret = ua_user_foreach_devices(g_user_h,
					_user_foreach_added_device_cb, NULL);

	msg(" - ua_user_foreach_devices() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_select_device(MManager *mm, struct menu_data *menu)
{
	GSList *iter = g_device_list;

	int id = 0;
	int selected_id = 0;

	if (strlen(g_selected_device_idx))
		selected_id = (unsigned char)strtol(g_selected_device_idx, NULL, 10);

	if (selected_id <= 0) {
		msg("Please select device first");
		return RET_SUCCESS;
	}

	while (NULL != iter) {
		GSList *next = iter->next;
		ua_device_h *handle = iter->data;
		if (handle && ++id == selected_id) {
			g_device_h = handle; /* Make selected device as a current one */
			uat_update_device_info(); /* Update device info. */
			msg("[%d] device selected", selected_id);
			return RET_SUCCESS;
		}
		iter = next;
	}

	return RET_SUCCESS;
}

static struct menu_data menu_ua_user_create[] = {
	{ "1", "account",
		NULL, NULL, g_user_account_str },
	{ "2", "run", NULL,
		run_ua_user_create, NULL },
	{ NULL, NULL, },
};

static struct menu_data menu_ua_service_get_user_by_account[] = {
	{ "1", "account",
		NULL, NULL, g_user_account_str},
	{ "2", "run", NULL,
		run_ua_service_get_user_by_account, NULL },
	{ NULL, NULL, },
};

struct menu_data menu_sel_added_user[] = {
	{ "1", "User list", NULL,
		run_ua_user_foreach, g_selected_user_idx},
	{ "2", "Apply", NULL, uat_select_user, NULL },
	{ NULL, NULL, },
};

struct menu_data menu_ua_user_added_devlist[] = {
	{ "1", "Device list", NULL,
		run_ua_user_foreach_devices, g_selected_device_idx},
	{ "2", "Apply", NULL, run_select_device, NULL },
	{ NULL, NULL, },
};

struct menu_data menu_ua_users[] = {
	{ "1", "ua_user_create",
		menu_ua_user_create, NULL, NULL },
	{ "2", "ua_user_destroy",
		NULL, run_ua_user_destroy, NULL },
	{ "4", "ua_user_add",
		NULL, run_ua_user_add, NULL },
	{ "5", "ua_user_remove",
		NULL, run_ua_user_remove, NULL},
	{ "6", "ua_get_default_user",
		NULL, run_ua_get_default_user, NULL },
	{ "7", "ua_service_get_user_by_account",
		menu_ua_service_get_user_by_account, NULL, NULL },
	{ "8", ANSI_COLOR_LIGHTMAGENTA "ua_foreach_users" ANSI_COLOR_NORMAL,
		menu_sel_added_user, NULL, NULL },
	{ "11", ANSI_COLOR_LIGHTGREEN "ua_user_add_device" ANSI_COLOR_NORMAL,
		NULL, run_ua_user_add_device, NULL },
	{ "12", ANSI_COLOR_LIGHTRED "ua_user_remove_device" ANSI_COLOR_NORMAL,
		NULL, run_ua_user_remove_device, NULL },
	{ "14", ANSI_COLOR_LIGHTMAGENTA "ua_user_foreach_devices" ANSI_COLOR_NORMAL,
		menu_ua_user_added_devlist, NULL, NULL },
	{ NULL, NULL, },
};

struct menu_data menu_sel_user[] = {
	{ "1", "User list", NULL, run_ua_user_foreach, g_selected_user_idx},
	{ "2", "Apply", NULL, uat_select_user , NULL },
	{ NULL, NULL, },
};
