/*
 * Copyright (c) 2021 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>

#include <system_info.h>
#include <glib.h>
#include <gio/gio.h>

#ifndef API
#define API __attribute__ ((visibility("default")))
#endif

#define UAM_SERVICE_MAX_STRING_LEN 50
#define UAM_USER_ACCOUNT_MAX_STRING_LEN 254
#define UAM_USER_NAME_MAX_STRING_LEN 254
#define UAM_MAC_ADDRESS_STRING_LEN 18
#define UAM_IP_ADDRESS_MAX_STRING_LEN 50
#define UAM_DEVICE_ID_MAX_STRING_LEN 50

typedef enum {
	UA_SENSOR_BT = 0x00000001,
	UA_SENSOR_BLE = 0x00000002,
	UA_SENSOR_WIFI = 0x00000004,
	UA_SENSOR_MOTION = 0x00000008,
	UA_SENSOR_LIGHT = 0x00000010,
	UA_SENSOR_AUDIO = 0x00000020,
	UA_SENSOR_MAX,
	UA_SENSOR_ALL = 0xFFFFFFFF
} ua_sensor_type_e;

typedef enum {
	UAM_ERROR_NONE = TIZEN_ERROR_NONE,
	UAM_ERROR_INVALID_PARAMETER = TIZEN_ERROR_INVALID_PARAMETER,
	UAM_ERROR_OUT_OF_MEMORY = TIZEN_ERROR_OUT_OF_MEMORY,
	UAM_ERROR_PERMISSION_DENIED = TIZEN_ERROR_PERMISSION_DENIED,
	UAM_ERROR_RESOURCE_BUSY = TIZEN_ERROR_RESOURCE_BUSY,
	UAM_ERROR_TIMED_OUT = TIZEN_ERROR_TIMED_OUT,
	UAM_ERROR_NOW_IN_PROGRESS = TIZEN_ERROR_NOW_IN_PROGRESS,
	UAM_ERROR_NOT_SUPPORTED = TIZEN_ERROR_NOT_SUPPORTED,
} uam_error_e;

typedef enum {
	UAM_OS_TYPE_UNDEFINED = 0x00,
	UAM_OS_TYPE_TIZEN,
	UAM_OS_TYPE_ANDROID,
	UAM_OS_TYPE_IOS,
	UAM_OS_TYPE_INVALID
} uam_os_type_e;


typedef enum {
	UAM_TECH_TYPE_NONE = 0x00,
	UAM_TECH_TYPE_BT = 0x01,
	UAM_TECH_TYPE_BLE = 0x02,
	UAM_TECH_TYPE_WIFI = 0x04,
	UAM_TECH_TYPE_P2P = 0x08,
	UAM_TECH_TYPE_MAX
} uam_tech_type_e;

typedef struct {
	int result;
	void *data;
} uam_event_data_s;

typedef struct {
	guint id;
	void *cb;
	void *user_data;
} uam_event_handler_data_t;

typedef struct {
	char name[UAM_SERVICE_MAX_STRING_LEN];
	unsigned int presence_threshold;
	unsigned int absence_threshold;
	int app_num;
} uam_service_info_s;

typedef struct {
	char account[UAM_USER_ACCOUNT_MAX_STRING_LEN];
	char name[UAM_USER_NAME_MAX_STRING_LEN];
	int app_num;
} uam_user_info_s;

typedef struct {
	uam_os_type_e operating_system;
	uam_tech_type_e type;
	char mac[UAM_MAC_ADDRESS_STRING_LEN];
	char ipv4_addr[UAM_IP_ADDRESS_MAX_STRING_LEN];
	char device_id[UAM_DEVICE_ID_MAX_STRING_LEN];
	gboolean discriminant;
	unsigned long long last_seen;
	int app_num;
} uam_device_info_s;

typedef void (*uam_event_cb)(int event, uam_event_data_s *event_param, void *user_data);

static uam_event_handler_data_t *event_handler_data = NULL;

API int system_info_get_platform_bool(const char *key, bool *value)
{
	*value = TRUE;
	return SYSTEM_INFO_ERROR_NONE;
}
API int _uam_init(uam_event_cb cb, void *user_data)
{
	event_handler_data = g_malloc0(sizeof(uam_event_handler_data_t));
	if (event_handler_data) {
		event_handler_data->cb = cb;
		event_handler_data->user_data = user_data;
	}

	return UAM_ERROR_NONE;
}

API int _uam_deinit(void)
{
	g_free(event_handler_data);

	return UAM_ERROR_NONE;
}

API int _uam_get_registered_services(GPtrArray **service_list)
{
	uam_service_info_s *service_info = NULL;

	service_info = g_new0(uam_service_info_s, 1);
	if (service_info) {
		g_strlcpy(service_info->name, "ua.service.default", UAM_SERVICE_MAX_STRING_LEN);
		service_info->presence_threshold = 2;
		service_info->absence_threshold = 0;
		service_info->app_num = 1;

		g_ptr_array_add(*service_list, (gpointer)service_info);
	}

	return UAM_ERROR_NONE;
}

API int _uam_get_registered_users(GPtrArray **user_list)
{
	uam_user_info_s *user_info = NULL;

	user_info = g_new0(uam_user_info_s, 1);
	if (user_info) {
		g_strlcpy(user_info->name, "default.1", UAM_USER_NAME_MAX_STRING_LEN);
		g_strlcpy(user_info->account, "default.1@default.com", UAM_USER_ACCOUNT_MAX_STRING_LEN);
		user_info->app_num = 1;

		g_ptr_array_add(*user_list, (gpointer)user_info);
	}

	return UAM_ERROR_NONE;
}

API int _uam_request_get_devices(GPtrArray **devices_list)
{
	uam_device_info_s *device_info = NULL;

	device_info = g_new0(uam_device_info_s, 1);
	if (device_info) {
		device_info->operating_system = 0x02;
		device_info->type = 0x02;
		device_info->discriminant = 1;
		g_strlcpy(device_info->mac, "30:AB:6A:09:1B:4C", UAM_MAC_ADDRESS_STRING_LEN);
		g_strlcpy(device_info->device_id, "18:54:CF:06:37:FE", UAM_DEVICE_ID_MAX_STRING_LEN);
		g_strlcpy(device_info->ipv4_addr, "192.168.107.10", UAM_IP_ADDRESS_MAX_STRING_LEN);
		device_info->last_seen = 1844674407370955;
		device_info->app_num = 1;

		g_ptr_array_add(*devices_list, (gpointer)device_info);
	}

	return UAM_ERROR_NONE;
}

API int _uam_get_default_service(uam_service_info_s *service)
{
	g_strlcpy(service->name, "ua.service.default", UAM_SERVICE_MAX_STRING_LEN);
	service->presence_threshold = 2;
	service->absence_threshold = 0;
	service->app_num = 1;

	return UAM_ERROR_NONE;
}

API int _uam_get_default_user(uam_user_info_s *user)
{
	g_strlcpy(user->name, "default.1", UAM_USER_NAME_MAX_STRING_LEN);
	g_strlcpy(user->account, "default.1@default.com", UAM_USER_ACCOUNT_MAX_STRING_LEN);
	user->app_num = 1;

	return UAM_ERROR_NONE;
}

API int _uam_get_available_sensors(unsigned int *bitmask)
{
	*bitmask = UA_SENSOR_BT | UA_SENSOR_BLE | UA_SENSOR_WIFI;

	return UAM_ERROR_NONE;
}

API int _uam_start_search_active_devices(unsigned int bitmask, int detection_period)
{
	return UAM_ERROR_NONE;
}

API int _uam_stop_search_active_devices(unsigned int bitmask)
{
	return UAM_ERROR_NONE;
}

API int _uam_start_presence_detection(unsigned int bitmask,
	const char *service)
{
	return UAM_ERROR_NONE;
}

API int _uam_start_absence_detection(unsigned int bitmask,
	const char *service)
{
	return UAM_ERROR_NONE;
}

API int _uam_stop_presence_detection(unsigned int bitmask,
	const char *service)
{
	return UAM_ERROR_NONE;
}

API int _uam_stop_absence_detection(unsigned int bitmask,
	const char *service)
{
	return UAM_ERROR_NONE;
}

API int _uam_add_user(uam_user_info_s *user)
{
	return UAM_ERROR_NONE;
}

API int _uam_remove_user(char *account)
{
	return UAM_ERROR_NONE;
}

API int _uam_request_add_device(char *account,
	uam_device_info_s *device)
{
	return UAM_ERROR_NONE;
}

API int _uam_request_remove_device(char *account,
	uam_device_info_s *device)
{
	return UAM_ERROR_NONE;
}

API int _uam_request_get_user_devices(char *account,
	GPtrArray **devices_list)
{
	uam_device_info_s *device_info = NULL;

	device_info = g_new0(uam_device_info_s, 1);
	if (device_info) {
		device_info->operating_system = 0x02;
		device_info->type = 0x02;
		device_info->discriminant = 1;
		g_strlcpy(device_info->mac, "30:AB:6A:09:1B:4C", UAM_MAC_ADDRESS_STRING_LEN);
		g_strlcpy(device_info->device_id, "18:54:CF:06:37:FE", UAM_DEVICE_ID_MAX_STRING_LEN);
		g_strlcpy(device_info->ipv4_addr, "192.168.107.10", UAM_IP_ADDRESS_MAX_STRING_LEN);
		device_info->last_seen = 1844674407370955;
		device_info->app_num = 1;

		g_ptr_array_add(*devices_list, (gpointer)device_info);
	}

	return UAM_ERROR_NONE;
}

API int _uam_register_service(uam_service_info_s *service)
{
	return UAM_ERROR_NONE;
}

API int _uam_update_service(uam_service_info_s *service)
{
	return UAM_ERROR_NONE;
}

API int _uam_unregister_service(uam_service_info_s *service)
{
	return UAM_ERROR_NONE;
}

API int _uam_service_add_user(const char *service, const char *account)
{
	return UAM_ERROR_NONE;
}

API int _uam_service_remove_user(const char *service,
	const char *account)
{
	return UAM_ERROR_NONE;
}

API int _uam_request_get_service_users(char *svc_name,
	GPtrArray **users_list)
{
	uam_user_info_s *user_info = NULL;

	user_info = g_new0(uam_user_info_s, 1);
	if (user_info) {
		g_strlcpy(user_info->name, "default.1", UAM_USER_NAME_MAX_STRING_LEN);
		g_strlcpy(user_info->account, "default.1@default.com", UAM_USER_ACCOUNT_MAX_STRING_LEN);
		user_info->app_num = 1;

		g_ptr_array_add(*users_list, (gpointer)user_info);
	}

	return UAM_ERROR_NONE;
}

API int _uam_is_device_registered(uam_device_info_s *dev_info,
	gboolean *is_registered)
{
	*is_registered = true;

	return UAM_ERROR_NONE;
}

API int _uam_request_update_device(uam_device_info_s *device)
{
	return UAM_ERROR_NONE;
}

API int _uam_request_get_device_by_deviceid(const char *device_id,
	uam_tech_type_e tech_type, uam_device_info_s *device)
{
	device->operating_system = 0x02;
	device->type = 0x02;
	device->discriminant = 1;
	g_strlcpy(device->mac, "30:AB:6A:09:1B:4C", UAM_MAC_ADDRESS_STRING_LEN);
	g_strlcpy(device->device_id, "18:54:CF:06:37:FE", UAM_DEVICE_ID_MAX_STRING_LEN);
	g_strlcpy(device->ipv4_addr, "192.168.107.10", UAM_IP_ADDRESS_MAX_STRING_LEN);
	device->last_seen = 1844674407370955;
	device->app_num = 1;

	return UAM_ERROR_NONE;
}

API int _uam_request_add_sensor(unsigned int bitmask)
{
	return UAM_ERROR_NONE;
}

API int _uam_request_remove_sensor(unsigned int bitmask)
{
	return UAM_ERROR_NONE;
}
