/*
 * Copyright (c) 2021 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <system_info.h>
#include <glib.h>
#include <dlog.h>
#include <user-awareness.h>

#include "utc-user-awareness-common.h"
#include "assert_local.h"

GMainLoop *g_pMainLoop;
guint g_nTimeoutId;
int g_nCallbackRet;
bool g_bInitialized;
bool g_bFeatureUA;

static bool __ua_foreach_service_cb(
	ua_service_h service_handle, void *user_data)
{
	g_nCallbackRet = UA_ERROR_NONE;

	return true;
}

/**
 * @function      utc_startup
 * @description   Called before each test
 * @parameter     NA
 * @return        NA
 */
void utc_startup(void)
{
	g_bInitialized = false;

	g_bFeatureUA = ua_check_feature_supported(UA_FEATURE);
	if (g_bFeatureUA) {
		if (UA_ERROR_NONE == ua_initialize())
			g_bInitialized = true;
	} else {
		DEBUG_LOG("Feature is not supported");
	}
}

/**
 * @function      utc_cleanup
 * @description   Called after each test
 * @parameter     NA
 * @return        NA
 */
void utc_cleanup(void)
{
	if (g_bFeatureUA && g_bInitialized)
		ua_deinitialize();
}

/**
 * @testcase		utc_ua_foreach_service_p
 * @since_tizen		6.5
 * @type			Positive
 * @description		Get UA services.
 * @scenario		Get UA services.
 *                  If already created, removes it and creates again.
 */
int utc_ua_foreach_service_p(void)
{
	int ret = UA_ERROR_NONE;

	CHECK_INIT_RETURN(g_bInitialized);

	if (!g_bFeatureUA) {
		ret = ua_foreach_service(NULL, NULL);
		CHECK_RETURN("ua_foreach_service", ret, UA_ERROR_NOT_SUPPORTED);
		return 0;
	}

	ret = ua_foreach_service(__ua_foreach_service_cb, NULL);
	CHECK_RETURN("ua_foreach_service", ret, UA_ERROR_NONE);

	return 0;
}

/**
 * @testcase		utc_ua_foreach_service_n
 * @since_tizen		6.5
 * @type			Negative
 * @description		Get UA services.
 * @scenario		Get UA services.
 *                  Ensure return value should be UA_ERROR_INVALID_PARAMETER.
 */
int utc_ua_foreach_service_n(void)
{
	int ret = UA_ERROR_NONE;

	CHECK_INIT_RETURN(g_bInitialized);

	ret = ua_foreach_service(NULL, NULL);
	if (!g_bFeatureUA) {
		CHECK_RETURN("ua_foreach_service", ret, UA_ERROR_NOT_SUPPORTED);
	} else {
		CHECK_RETURN("ua_foreach_service", ret, UA_ERROR_INVALID_PARAMETER);
	}

	return 0;
}

int utc_ua_get_default_service_p(void)
{
	int ret = UA_ERROR_NONE;
	ua_service_h service_h = NULL;

	CHECK_INIT_RETURN(g_bInitialized);

	if (!g_bFeatureUA) {
		ret = ua_get_default_service(NULL);
		CHECK_RETURN("ua_get_default_service", ret, UA_ERROR_NOT_SUPPORTED);
		return 0;
	}

	ret = ua_get_default_service(&service_h);
	CHECK_RETURN("ua_get_default_service", ret, UA_ERROR_NONE);

	return 0;
}

int utc_ua_get_default_service_n(void)
{
	int ret = UA_ERROR_NONE;

	CHECK_INIT_RETURN(g_bInitialized);

	ret = ua_get_default_service(NULL);
	if (!g_bFeatureUA) {
		CHECK_RETURN("ua_get_default_service", ret, UA_ERROR_NOT_SUPPORTED);
	} else {
		CHECK_RETURN("ua_get_default_service", ret, UA_ERROR_INVALID_PARAMETER);
	}

	return 0;
}

int utc_ua_get_service_by_name_p(void)
{
	int ret = UA_ERROR_NONE;
	ua_service_h service_h = NULL;
	char service_str[] = {"ua.service.default"};

	CHECK_INIT_RETURN(g_bInitialized);

	if (!g_bFeatureUA) {
		ret = ua_get_service_by_name(NULL, NULL);
		CHECK_RETURN("ua_get_service_by_name", ret, UA_ERROR_NOT_SUPPORTED);
		return 0;
	}

	ret = ua_get_service_by_name(service_str, &service_h);
	CHECK_RETURN("ua_get_service_by_name", ret, UA_ERROR_NONE);

	return 0;
}

int utc_ua_get_service_by_name_n(void)
{
	int ret = UA_ERROR_NONE;

	CHECK_INIT_RETURN(g_bInitialized);

	ret = ua_get_service_by_name(NULL, NULL);
	if (!g_bFeatureUA) {
		CHECK_RETURN("ua_get_service_by_name", ret, UA_ERROR_NOT_SUPPORTED);
	} else {
		CHECK_RETURN("ua_get_service_by_name", ret, UA_ERROR_INVALID_PARAMETER);
	}

	return 0;
}
